from typing import Callable, Optional, Union
import uuid
from PySide6.QtWidgets import  QFrame, QHBoxLayout, QSizePolicy, QWidget
from PySide6.QtCore import  Qt, Signal, QEvent

from qtmui.hooks import State, useEffect

from ..system.color_manipulator import alpha

from .li_base import LiBase

from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from ..widget_base import PyWidgetBase
from ..typography import Typography

class Li(LiBase, PyWidgetBase):

    selectedChange = Signal(object)

    def __init__(self, 
                 alignItems='center',  # Căn chỉnh items
                 autoFocus=False,  # Tự động focus khi mount
                 children=None,  # Nội dung của MenuItem
                 classes=None,  # Ghi đè hoặc mở rộng các style
                 component=QFrame,  # Thành phần dùng cho root node
                 dense=False,  # Sử dụng padding dọc nhỏ gọn
                 disabled=False,  # Vô hiệu hóa component
                 disableGutters=False,  # Loại bỏ padding trái/phải
                 divider=False,  # Thêm viền dưới
                 focusVisibleClassName=None,  # Lớp CSS khi focus
                 key = None,
                 minHeight=None,
                 onChange: Callable = None,
                 onClick: Callable = None,
                 selected=False,  # Sử dụng style khi được chọn
                 sx=None,  # Hệ thống prop cho overrides và styles
                 visible: bool = True,  # Hệ thống prop cho overrides và styles
                 size="medium",  # Hệ thống prop cho overrides và styles
                 parent=None,  # Parent widget
                 selectedKey: State=None,  # Parent widget
                text: Optional[Union[State, str, Callable]] = None,
                 textAlign: str = 'left',  # Parent widget
                 **kwargs):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))

        self._alignItems = alignItems
        self._autoFocus = autoFocus
        self._children = children
        self._classes = classes
        self._component = component
        self._dense = dense
        self._disabled = disabled
        self._disableGutters = disableGutters
        self._divider = divider
        self._minHeight = minHeight
        self._onChange = onChange or print
        self._focusVisibleClassName = focusVisibleClassName
        self._selected = selected
        self._sx = sx

        self._visible = visible
        self._size = size
        self._text = text
        self._textAlign = textAlign

        self._key = key
        self._onListItemButtonClick = onClick
        self._selectedKeys = selectedKey
        self._searchdKeys = None

        self.__init_ui()


    def __init_ui(self):

        self.setDisabled(self._disabled)

        self.setCursor(Qt.CursorShape.PointingHandCursor)

        if self._selected:
            self.setProperty("selected", "true")

        if self._textAlign in ["left", "center", "right"]:
            self._textAlign = f"text-align: {self._textAlign};"

        if self._minHeight:
            self.setMinimumHeight(self._minHeight)

        if self._selectedKeys:
            if isinstance(self._selectedKeys, State):
                self._selectedKeys.valueChanged.connect(self._set_selected)

        if self._children:
            self.setLayout(QHBoxLayout())
            # self.layout().setContentsMargins(6,6,6,6)
            self.layout().setContentsMargins(0,0,0,0)
            self.layout().setAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
            self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)
  
            if isinstance(self._children, list):
                for child in self._children:
                    self.layout().addWidget(child)
            elif isinstance(self._children, QWidget):
                self.layout().addWidget(self._children)
            if self._text:
                self.layout().addWidget(Typography(text=self._text, variant="button"))
        else:
            self.setText(self._text)

        PyWidgetBase._installTooltipFilter(self)

        for widget in self.findChildren(QWidget):
            widget.installEventFilter(self)  # Bắt sự kiện từ con

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

    def _set_on_changed(self, onChange: Callable):
        self._onChange = onChange
        self.selectedChange.connect(self._onChange)

    def _set_selected_keys(self, selectedKeys: State):
        self._selectedKeys = selectedKeys
        self._selectedKeys.valueChanged.connect(self._set_selected)
        self._set_selected(selectedKeys.value)

    # def _set_search_keys(self, _searchKeys: State):
    #     self._searchKeys = _searchKeys
    #     self._searchKeys.valueChanged.connect(self._set_search_results)

    def _set_selected(self, keys):
        if isinstance(keys, list):
            if self._key in keys:
                self.setProperty("selected", "true")
            else:
                self.setProperty("selected", "false")
        else:
            # print('kkeys____________', keys)
            if self._key == keys:
                self.setProperty("selected", "true")
            else:
                self.setProperty("selected", "false")
        self._set_stylesheet()

    def _set_stylesheet(self):
        theme = useTheme()
        component_styles = theme.components

        PyMenuItem = component_styles[f"PyMenuItem"].get("styles")
        PyMenuItem_root_qss = get_qss_style(PyMenuItem["root"])
        PyMenuItem_root_slot_hover_qss = get_qss_style(PyMenuItem["root"]["slots"]["hover"])
        PyMenuItem_root_slot_selected_qss = get_qss_style(PyMenuItem["root"]["slots"]["selected"])
        PyMenuItem_root_slot_selected_hover_qss = get_qss_style(PyMenuItem["root"]["slots"]["selected"]["hover"])

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx


        stylesheet = f"""
            #{self.objectName()} {{
                {PyMenuItem_root_qss}
            }}
            #{self.objectName()}::hover {{
                {PyMenuItem_root_slot_hover_qss}
            }}
            #{self.objectName()}[selected=true] {{
                {PyMenuItem_root_slot_selected_qss}
            }}
            #{self.objectName()}[selected=true]::hover {{
                {PyMenuItem_root_slot_selected_hover_qss}
            }}

            {sx_qss}
        """

        self.setStyleSheet(stylesheet)


    # def _set_search_results(self, keys):
    #     if isinstance(keys, list):
    #         if self._key in keys:
    #             self.setProperty("search", "true")
    #         else:
    #             self.setProperty("search", "false")
    #     else:
    #         if self._key == keys:
    #             self.setProperty("search", "true")
    #         else:
    #             self.setProperty("search", "false")
    #     self._set_stylesheet()


    def eventFilter(self, obj, event):
        if event.type() == QEvent.MouseButtonPress:
            # print("🖱️ Mouse click on child:", obj)
            # Thực hiện logic ở đây, ví dụ chọn dòng, đổi màu...
            if self._onListItemButtonClick:
                self._onListItemButtonClick({"label": self._text or self._key, "value": self._key})
            self.selectedChange.emit({"label": self._text or self._key, "value": self._key})
            return False  # cho sự kiện đi tiếp (QCheckBox vẫn nhận)
        # elif event.type() == QEvent.Enter:
        #     print("🟩 Mouse entered child:", obj)
        # elif event.type() == QEvent.Leave:
        #     print("🟥 Mouse left child:", obj)
        return super().eventFilter(obj, event)
    


from typing import Optional, Union, Dict, Callable, List
from PySide6.QtWidgets import QVBoxLayout, QWidget, QApplication, QFrame, QSizePolicy, QSplitter, QGraphicsDropShadowEffect, QSplitterHandle
from PySide6.QtCore import Qt, QPoint, QEvent
from PySide6.QtGui import QColor
from qtmui.material.styles import useTheme
from qtmui.i18n.use_translation import i18n
from qtmui.hooks import State, useEffect
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..utils.validate_params import _validate_param

class Drawer(QSplitter):
    """
    A component that renders a drawer sliding in from a specified side.

    The `Drawer` component is used to display a panel that slides in from one side of the
    screen, supporting all props of the Material-UI `Drawer` component, as well as additional
    custom props.

    Parameters
    ----------
    parent : QWidget or None, optional
        The parent widget. Default is None.
    title : State or str, optional
        The title of the drawer (custom feature, not part of Material-UI). Default is "Drawer".
        Can be a `State` object for dynamic updates.
    anchor : State or str, optional
        Side from which the drawer will appear ("bottom", "left", "right", "top"). Default is "left".
        Can be a `State` object for dynamic updates.
    anchorEl : State, QWidget, or None, optional
        The anchor element to position the drawer relative to (custom feature, not part of Material-UI).
        Default is None (uses main window).
        Can be a `State` object for dynamic updates.
    children : State, QWidget, List[QWidget], or None, optional
        The content of the component. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    elevation : State or int, optional
        The elevation of the drawer. Default is 16.
        Can be a `State` object for dynamic updates.
    hideBackdrop : State or bool, optional
        If True, the backdrop is not rendered. Default is False.
        Can be a `State` object for dynamic updates.
    ModalProps : State, dict, or None, optional
        Props applied to the Modal element. Default is {}.
        Can be a `State` object for dynamic updates.
    onClose : State, Callable, or None, optional
        Callback fired when the component requests to be closed. Default is None.
        Signature: function(event: object, reason: string) => void
        Can be a `State` object for dynamic updates.
    open : State or bool, optional
        If True, the drawer is shown. Default is False.
        Can be a `State` object for dynamic updates.
    PaperProps : State, dict, or None, optional
        Props applied to the Paper element. Default is {}.
        Deprecated: Use `slotProps.paper` instead.
        Can be a `State` object for dynamic updates.
    SlideProps : State, dict, or None, optional
        Props applied to the Slide element. Default is {}.
        Deprecated: Use `slotProps.transition` instead.
        Can be a `State` object for dynamic updates.
    slotProps : State, dict, or None, optional
        Props used for each slot inside (backdrop, docked, paper, root, transition). Default is {}.
        Can be a `State` object for dynamic updates.
    slots : State, dict, or None, optional
        Components used for each slot inside (backdrop, docked, paper, root, transition). Default is {}.
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    transitionDuration : State, int, dict, or None, optional
        Duration for the transition in milliseconds. Default is
        { "enter": theme.transitions.duration.enteringScreen, "exit": theme.transitions.duration.leavingScreen }.
        Can be a `State` object for dynamic updates.
    variant : State, str, or None, optional
        The drawer variant ("permanent", "persistent", "temporary"). Default is "temporary".
        Can be a `State` object for dynamic updates.
    minWidth : State or int, optional
        Minimum width of the drawer (custom feature, not part of Material-UI). Default is 300.
        Can be a `State` object for dynamic updates.
    maxWidth : State or int, optional
        Maximum width of the drawer (custom feature, not part of Material-UI). Default is 600.
        Can be a `State` object for dynamic updates.
    minHeight : State or int, optional
        Minimum height of the drawer (custom feature, not part of Material-UI). Default is 200.
        Can be a `State` object for dynamic updates.
    maxHeight : State or int, optional
        Maximum height of the drawer (custom feature, not part of Material-UI). Default is 400.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class, supporting
        props of the native component (e.g., style, className).

    Attributes
    ----------
    VALID_ANCHOR : list[str]
        Valid values for `anchor`: ["bottom", "left", "right", "top"].
    VALID_VARIANT : list[str]
        Valid values for `variant`: ["permanent", "persistent", "temporary"].

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `style`, `className`).
    - The `title`, `minWidth`, `maxWidth`, `minHeight`, `maxHeight` parameters are custom features,
      not part of Material-UI's `Drawer`.
    - The `PaperProps` and `SlideProps` props are deprecated; use `slotProps.paper` and
      `slotProps.transition` instead.
    - The `onClose` callback receives an event and a reason ("escapeKeyDown", "backdropClick").

    Demos:
    - Drawer: https://qtmui.com/material-ui/qtmui-drawer/

    API Reference:
    - Drawer API: https://qtmui.com/material-ui/api/drawer/
    """

    VALID_ANCHOR = ["bottom", "left", "right", "top"]
    VALID_VARIANT = ["permanent", "persistent", "temporary"]

    def __init__(
        self,
        parent: Optional[QWidget] = None,
        title: Optional[Union[State, str, Callable]] = "Drawer",
        anchor: Union[State, str] = "right",
        anchorEl: Optional[Union[State, QWidget]] = None,
        children: Optional[Union[State, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        elevation: Union[State, int] = 16,
        hideBackdrop: Union[State, bool] = False,
        ModalProps: Optional[Union[State, Dict]] = None,
        onClose: Optional[Union[State, Callable]] = None,
        open: Union[State, bool] = False,
        PaperProps: Optional[Union[State, Dict]] = None,
        SlideProps: Optional[Union[State, Dict]] = None,
        slotProps: Optional[Union[State, Dict]] = None,
        slots: Optional[Union[State, Dict]] = None,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        transitionDuration: Optional[Union[State, int, Dict]] = None,
        variant: Optional[Union[State, str]] = "temporary",
        minWidth: Optional[Union[State, int]] = None,
        maxWidth: Optional[Union[State, int]] = None,
        minHeight: Optional[Union[State, int]] = None,
        maxHeight: Optional[Union[State, int]] = None,
        *args,
        **kwargs
    ):
        super().__init__(parent or QApplication.instance().mainWindow)

        self.setObjectName(f"PyDrawer")

        # Initialize theme
        self.theme = useTheme()

        # Store widget references to prevent Qt deletion
        self._widget_references = []

        # Set properties with validation
        self._set_title(title)
        self._set_anchor(anchor)
        self._set_anchorEl(anchorEl)
        self._set_children(children)
        self._set_classes(classes)
        self._set_elevation(elevation)
        self._set_hideBackdrop(hideBackdrop)
        self._set_ModalProps(ModalProps or {})
        self._set_onClose(onClose)
        self._set_open(open)
        self._set_PaperProps(PaperProps or {})
        self._set_SlideProps(SlideProps or {})
        self._set_slotProps(slotProps or {})
        self._set_slots(slots or {})
        self._set_sx(sx)
        # self._set_transitionDuration(transitionDuration or {
        #     "enter": self.theme.transitions.duration.enteringScreen,
        #     "exit": self.theme.transitions.duration.leavingScreen
        # })
        self._set_variant(variant)
        
        self._set_minWidth(minWidth)
        self._set_maxWidth(maxWidth)
        self._set_minHeight(minHeight)
        self._set_maxHeight(maxHeight)

        self._user_dragging = False

        # Setup UI
        self._init_ui()


    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.drawer", param_name="title", supported_signatures=Union[State, str, Callable])
    def _set_title(self, value):
        """Assign value to title (custom feature)."""
        self._title = value

    def _get_title(self):
        """Get the title value."""
        return self._title.value if isinstance(self._title, State) else self._title

    @_validate_param(file_path="qtmui.material.drawer", param_name="anchor", supported_signatures=Union[State, str], valid_values=VALID_ANCHOR)
    def _set_anchor(self, value):
        """Assign value to anchor."""
        self._anchor = value

    def _get_anchor(self):
        """Get the anchor value."""
        return self._anchor.value if isinstance(self._anchor, State) else self._anchor

    @_validate_param(file_path="qtmui.material.drawer", param_name="anchorEl", supported_signatures=Union[State, QWidget, type(None)])
    def _set_anchorEl(self, value):
        """Assign value to anchorEl (custom feature)."""
        self._anchorEl = value

    def _get_anchorEl(self):
        """Get the anchorEl value."""
        return self._anchorEl.value if isinstance(self._anchorEl, State) else self._anchorEl

    # @_validate_param(file_path="qtmui.material.drawer", param_name="children", supported_signatures=Union[State, QWidget, List[QWidget], type(None)])
    def _set_children(self, value):
        """Assign value to children and store widget references."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.drawer", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.drawer", param_name="elevation", supported_signatures=Union[State, int])
    def _set_elevation(self, value):
        """Assign value to elevation."""
        self._elevation = value

    def _get_elevation(self):
        """Get the elevation value."""
        return self._elevation.value if isinstance(self._elevation, State) else self._elevation

    @_validate_param(file_path="qtmui.material.drawer", param_name="hideBackdrop", supported_signatures=Union[State, bool])
    def _set_hideBackdrop(self, value):
        """Assign value to hideBackdrop."""
        self._hideBackdrop = value

    def _get_hideBackdrop(self):
        """Get the hideBackdrop value."""
        return self._hideBackdrop.value if isinstance(self._hideBackdrop, State) else self._hideBackdrop

    @_validate_param(file_path="qtmui.material.drawer", param_name="ModalProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_ModalProps(self, value):
        """Assign value to ModalProps."""
        self._ModalProps = value

    def _get_ModalProps(self):
        """Get the ModalProps value."""
        return self._ModalProps.value if isinstance(self._ModalProps, State) else self._ModalProps

    @_validate_param(file_path="qtmui.material.drawer", param_name="onClose", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClose(self, value):
        """Assign value to onClose."""
        self._onClose = value

    def _get_onClose(self):
        """Get the onClose value."""
        return self._onClose.value if isinstance(self._onClose, State) else self._onClose

    @_validate_param(file_path="qtmui.material.drawer", param_name="open", supported_signatures=Union[State, bool])
    def _set_open(self, value):
        """Assign value to open."""
        self._open = value

    def _get_open(self):
        """Get the open value."""
        return self._open.value if isinstance(self._open, State) else self._open

    @_validate_param(file_path="qtmui.material.drawer", param_name="PaperProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_PaperProps(self, value):
        """Assign value to PaperProps."""
        self._PaperProps = value

    def _get_PaperProps(self):
        """Get the PaperProps value."""
        return self._PaperProps.value if isinstance(self._PaperProps, State) else self._PaperProps

    @_validate_param(file_path="qtmui.material.drawer", param_name="SlideProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_SlideProps(self, value):
        """Assign value to SlideProps."""
        self._SlideProps = value

    def _get_SlideProps(self):
        """Get the SlideProps value."""
        return self._SlideProps.value if isinstance(self._SlideProps, State) else self._SlideProps

    @_validate_param(file_path="qtmui.material.drawer", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        """Assign value to slotProps."""
        self._slotProps = value

    def _get_slotProps(self):
        """Get the slotProps value."""
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.drawer", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        """Assign value to slots."""
        self._slots = value

    def _get_slots(self):
        """Get the slots value."""
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.drawer", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.drawer", param_name="transitionDuration", supported_signatures=Union[State, int, Dict, type(None)])
    def _set_transitionDuration(self, value):
        """Assign value to transitionDuration."""
        self._transitionDuration = value

    def _get_transitionDuration(self):
        """Get the transitionDuration value."""
        return self._transitionDuration.value if isinstance(self._transitionDuration, State) else self._transitionDuration

    @_validate_param(file_path="qtmui.material.drawer", param_name="variant", supported_signatures=Union[State, str, type(None)], valid_values=VALID_VARIANT)
    def _set_variant(self, value):
        """Assign value to variant."""
        self._variant = value

    def _get_variant(self):
        """Get the variant value."""
        return self._variant.value if isinstance(self._variant, State) else self._variant

    @_validate_param(file_path="qtmui.material.drawer", param_name="minWidth", supported_signatures=Union[State, int, None])
    def _set_minWidth(self, value):
        """Assign value to minWidth (custom feature)."""
        self._minWidth = value

    def _get_minWidth(self):
        """Get the minWidth value."""
        return self._minWidth.value if isinstance(self._minWidth, State) else self._minWidth

    @_validate_param(file_path="qtmui.material.drawer", param_name="maxWidth", supported_signatures=Union[State, int, None])
    def _set_maxWidth(self, value):
        """Assign value to maxWidth (custom feature)."""
        self._maxWidth = value

    def _get_maxWidth(self):
        """Get the maxWidth value."""
        return self._maxWidth.value if isinstance(self._maxWidth, State) else self._maxWidth

    @_validate_param(file_path="qtmui.material.drawer", param_name="minHeight", supported_signatures=Union[State, int, None])
    def _set_minHeight(self, value):
        """Assign value to minHeight (custom feature)."""
        self._minHeight = value

    def _get_minHeight(self):
        """Get the minHeight value."""
        return self._minHeight.value if isinstance(self._minHeight, State) else self._minHeight

    @_validate_param(file_path="qtmui.material.drawer", param_name="maxHeight", supported_signatures=Union[State, int, None])
    def _set_maxHeight(self, value):
        """Assign value to maxHeight (custom feature)."""
        self._maxHeight = value

    def _get_maxHeight(self):
        """Get the maxHeight value."""
        return self._maxHeight.value if isinstance(self._maxHeight, State) else self._maxHeight

    def __setupStates(self):
        if isinstance(self._minWidth, State):
            self._minWidth.valueChanged.connect(self.__setupSize)
        if isinstance(self._minHeight, State):
            self._minHeight.valueChanged.connect(self.__setupSize)
        if isinstance(self._maxWidth, State):
            self._maxWidth.valueChanged.connect(self.__setupSize)
        if isinstance(self._maxHeight, State):
            self._maxHeight.valueChanged.connect(self.__setupSize)

    def _init_ui(self):
        self.setObjectName("PyDrawer")
        # self.setWindowFlags(Qt.Window | Qt.FramelessWindowHint) # xài ok
        self.setWindowFlags(Qt.FramelessWindowHint)
        
        # Cho phép nền trong suốt
        self.setAttribute(Qt.WA_TranslucentBackground, True)
        self.setAutoFillBackground(False)

        self._frm_content = QFrame(self)
        self._frm_content.setObjectName("contentFrame")
        self._frm_content.setStyleSheet("#contentFrame { border: 1px solid red; }")
        # self._frm_content.setFrameShape(QFrame.StyledPanel)
        self._contentLayout = QVBoxLayout(self._frm_content)
        
        self._overlay = QWidget(self)
        self._overlay.setAttribute(Qt.WA_TransparentForMouseEvents, True)
        self._overlay.setAutoFillBackground(False)
        self._overlay.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self._overlay.setStyleSheet("background-color: transparent;")
        
        self.setHandleWidth(0)

        # Cho phép or không cho widget collapse
        self.setChildrenCollapsible(True)  # False để không cho widget collapse hết
        self.setOpaqueResize(True)  # True để thay đổi kích thước ngay khi kéo

        self.__setupSize()

        # Thêm children widget vào layout của Drawer
        if self._children is not None:
            if isinstance(self._children, list):
                for widget in self._children:
                    self._contentLayout.addWidget(widget)
            else:
                if isinstance(self._children, QWidget):
                    self._contentLayout.addWidget(self._children)

        if not isinstance(self._open, State):
            raise ValueError("The attribute 'open' only accepts input in the form of State.")
        
        self._open.valueChanged.connect(self._show)

        # Cài đặt event filter để theo dõi sự kiện resize và move
        self.installEventFilter(self)
        if self._anchorEl and isinstance(self._anchorEl, QWidget):
            self._anchorEl.installEventFilter(self)
        QApplication.instance().mainWindow.installEventFilter(self)

        self.theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        self._set_stylesheet()

        i18n.langChanged.connect(self.reTranslation)
        
        self.handle(1).installEventFilter(self)
        # self.splitterMoved.connect(lambda: (self.refresh(), self.update()))
        # self.splitterMoved.connect(self.setShadowEffect)
        
        self.__setupStates()

    def __setupSize(self, value=None):
        # Tính toán vị trí của Drawer dựa trên anchor
        if self._anchor == "left":
            self.setOrientation(Qt.Orientation.Horizontal)
            self.addWidget(self._frm_content)
            self.addWidget(self._overlay)
            self.setContentsMargins(1,0,0,0)
            if self._minWidth is not None:
                self.setSizes([self._get_minWidth(), 50])
                self._frm_content.setMinimumWidth(self._get_minWidth())
            if self._maxWidth is not None:
                self._frm_content.setMaximumWidth(self._get_maxWidth())
        elif self._anchor == "right":
            self.setOrientation(Qt.Orientation.Horizontal)
            self.addWidget(self._overlay)
            self.addWidget(self._frm_content)
            # print('self._minWidth______________', self._minWidth)
            self.setContentsMargins(0,0,1,1)
            if self._minWidth is not None:
                self.setSizes([5000, self._get_minWidth()])
                self._frm_content.setMinimumWidth(self._get_minWidth())
            if self._maxWidth is not None:
                self._frm_content.setMaximumWidth(self._get_maxWidth())
        elif self._anchor == "top":
            self.setOrientation(Qt.Orientation.Vertical)
            self.addWidget(self._frm_content)
            self.addWidget(self._overlay)
            self.setContentsMargins(0,1,0,0)
            if self._minHeight is not None:
                self.setSizes([self._get_minHeight(), 50])
                self._frm_content.setMinimumHeight(self._get_minHeight())
            if self._maxHeight is not None:
                self._frm_content.setMaximumHeight(self._get_maxHeight())
        elif self._anchor == "bottom":
            self.setOrientation(Qt.Orientation.Vertical)
            self.addWidget(self._overlay)
            self.addWidget(self._frm_content)
            self.setContentsMargins(0,1,0,0) # cho cạnh ngay ngắn hơn
            if self._minHeight is not None:
                self.setSizes([50, self._get_minHeight()])
                self._frm_content.setMinimumHeight(self._get_minHeight())
            if self._maxHeight is not None:
                self._frm_content.setMaximumHeight(self._get_maxHeight())
        self.update_position()


    def _set_stylesheet(self):
        self.theme = useTheme()
        component_styles = self.theme.components

        ownerState = {
            "anchor": self._anchor,
            "variant": self._variant,
        }

        PyDrawer_root = component_styles[f"PyDrawer"].get("styles")["root"](ownerState)
        PyDrawer_contentFrame = component_styles[f"PyDrawer"].get("styles")["contentFrame"](ownerState)
        PyDrawer_root_qss = get_qss_style(PyDrawer_root)
        PyDrawer_contentFrame_qss = get_qss_style(PyDrawer_contentFrame)


        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx)
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx)
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx
        
        self.setStyleSheet(
            f"""
                #PyDrawer {{
                    {PyDrawer_root_qss}
                    {sx_qss}
                    
                }}
                #contentFrame {{
                    {PyDrawer_contentFrame_qss}
                }}
            """
        )
        
        
        self.setShadowEffect()


    def setShadowEffect(self, blurRadius=60, offset=(0, 10), color=QColor(0, 0, 0, 50)):
        """ add shadow to dialog """
        # print("Setting shadow effect")
        try:
            shadowEffect = QGraphicsDropShadowEffect(self._frm_content)
            shadowEffect.setBlurRadius(blurRadius)
            shadowEffect.setOffset(*offset)
            shadowEffect.setColor(color)
            self._frm_content.setGraphicsEffect(None)
            self._frm_content.setGraphicsEffect(shadowEffect)
        except Exception as e:
            pass

    def reTranslation(self):
        if isinstance(self._title, Callable):
            pass

    def _show(self, state: bool):
        # Cập nhật vị trí của Drawer trước khi hiển thị
        if state and not self.isVisible():
            # self.update_position()
            self.move(-30000, -30000)
            self.setVisible(True)
            self.update_position()
            # self.raise_()
        else:
            self.setVisible(False)

    def _close(self):
        self.setVisible(False)
        if self._onClose:
            self._onClose()

    def hideEvent(self, event):
        if self._onClose:
            self._onClose()
        return super().hideEvent(event)

    # def showDrawer(self):
    #     # print('showwwwwwwwwww drawerrrrrrrrrr')
    #     # Cập nhật vị trí của Drawer trước khi hiển thị
    #     # self.move(-30000, -30000)
    #     self.setVisible(True)
    #     # self.update_position()
    #     QTimer.singleShot(100, self.update_position)
        
    #     # self.raise_()


    def update_position(self):
        # Lấy vị trí và kích thước của anchorEl
        if not self._anchorEl:
            self._anchorEl = QApplication.instance().mainWindow
        # self._anchorEl.update()
        popup_position = self._anchorEl.mapToGlobal(QPoint(0, 0))
        self.setGeometry(popup_position.x(), popup_position.y(), self._anchorEl.width(), self._anchorEl.height())


    def eventFilter(self, source, event):
        if ((event.type() == QEvent.Resize) or (event.type() == QEvent.Move)) and source == self._anchorEl:
            if self.isVisible():
                # self.update_position()
                self.setVisible(False)
                
        if event.type() == QEvent.MouseButtonPress:
            self._user_dragging = True
            if not self._frm_content.underMouse() and not isinstance(source, QSplitterHandle):
                self._user_dragging = False
                self.hide()
                print('source_____________', source)
        elif event.type() == QEvent.MouseButtonRelease and self._user_dragging:
            self._user_dragging = False
            self.refresh()
            self.update()
            
        return super().eventFilter(source, event)
    



import uuid
from PySide6.QtWidgets import QVBoxLayout, QFrame, QSizePolicy
from PySide6.QtCore import Qt
from typing import Optional, Union, List, Callable

from qtmui.hooks import State

from ..typography import Typography
from ..box import Box

from qtmui.material.styles import useTheme

class AlignBox(Box):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

class TimelineContent(QFrame):
    def __init__(self,
                 children=None,  # Nội dung của component (node)
                 classes: dict = None,  # Ghi đè hoặc mở rộng các styles áp dụng cho component (dict)
                 sx: Union[List[Union[Callable, dict, bool]], Callable, dict] = None,  # Hệ thống prop cho phép ghi đè hệ thống hoặc thêm CSS styles
                text: Optional[Union[str, State, Callable]] = None,
                 ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))
        # self.setStyleSheet(f"#{self.objectName()} {{background-color: orange;}}")

        self._children = children
        self._classes = classes if classes is not None else {}
        self._sx = sx
        self._text = text

        self.theme = useTheme()

        # Khởi tạo giao diện
        self._initUI()

    def _initUI(self):
        """Khởi tạo giao diện của TimelineContent."""
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.layout().setAlignment(Qt.AlignTop)

        if self._text:
            self.content_widget = Typography(variant="body2", text=self._text)
            self.layout().addWidget(self.content_widget)
        else:
            if not isinstance(self._children, list):
                raise TypeError("One of the two values, text or children, must have a value.")
            
            new_children = []
            for child in self._children:
                new_children.append(AlignBox(
                    direction="row",
                    children=[child]
                ))
            self.content_widget = Box(
                sx={
                    "background-color": self.theme.palette.grey._300 if self.theme.palette.mode == "light" else self.theme.palette.grey._800,
                    "border-radius": 8
                },
                children=new_children
            )
            self.layout().addWidget(self.content_widget)




from dataclasses import dataclass, field
from typing import Callable, Optional, Union, Dict

from ....material.system.color_manipulator import get_contrast_text

# Định nghĩa các class và kiểu dữ liệu cần thiết

@dataclass
class CommonColors:
    black: str = "#000000"
    white: str = "#ffffff"

@dataclass
class Color:
    _50: str = ""
    _100: str = ""
    _200: str = ""
    _300: str = ""
    _400: str = ""
    _500: str = ""
    _600: str = ""
    _700: str = ""
    _800: str = ""
    _900: str = ""

@dataclass
class PaletteColor:
    light: Optional[str] = None
    lighter: Optional[str] = None
    main: str = ""
    dark: Optional[str] = None
    darker: Optional[str] = None
    contrastText: Optional[str] = None

@dataclass
class TypeText:
    primary: str = ""
    secondary: str = ""
    disabled: str = ""

@dataclass
class TypeAction:
    active: str = ""
    hover: str = ""
    hoverOpacity: float = 0.0
    selected: str = ""
    selectedOpacity: float = 0.0
    disabled: str = ""
    disabledBackground: str = ""
    disabledOpacity: float = 0.0
    focus: str = ""
    focusOpacity: float = 0.0
    activatedOpacity: float = 0.0

@dataclass
class TypeBackground:
    default: str = ""
    paper: str = ""
    notched: str = ""
    neutral: str = ""
    navigation: str = ""
    main: str = ""
    second: str = ""
    thirty: str = ""
    content: str = ""
    transparent: str = "transparent"
    
@dataclass
class Palette:
    common: CommonColors
    mode: str
    contrastThreshold: float
    tonalOffset: Union[float, Dict[str, float]]
    primary: PaletteColor
    secondary: PaletteColor
    error: PaletteColor
    warning: PaletteColor
    info: PaletteColor
    success: PaletteColor
    grey: Color
    text: TypeText
    divider: str
    action: TypeAction
    background: TypeBackground
    getContrastText: Callable[[str], str]
    augmentColor: Callable[[Dict], PaletteColor]

# Hàm để tạo ra các màu mặc định

def get_default_primary(mode: str = 'light') -> PaletteColor:
    if mode == 'dark':
        return PaletteColor(main="#90caf9", light="#e3f2fd", dark="#42a5f5")
    return PaletteColor(main="#1976d2", light="#63a4ff", dark="#004ba0")

def get_default_secondary(mode: str = 'light') -> PaletteColor:
    if mode == 'dark':
        return PaletteColor(main="#f48fb1", light="#f8bbd0", dark="#f06292")
    return PaletteColor(main="#dc004e", light="#ff5983", dark="#9a0036")

def get_default_error(mode: str = 'light') -> PaletteColor:
    if mode == 'dark':
        return PaletteColor(main="#ef5350", light="#e57373", dark="#d32f2f")
    return PaletteColor(main="#d32f2f", light="#ef5350", dark="#c62828")

# Hàm tính toán tỷ lệ tương phản


# Hàm để lấy màu chữ tương phản



# Hàm augmentColor

def augment_color(options: Dict) -> PaletteColor:
    color = options.get('color', {})
    if not color.get('main'):
        color['main'] = color.get(500, "")
    if not color['main']:
        raise ValueError(f"MUI: Invalid color. The color object needs to have a `main` property.")
    return PaletteColor(
        main=color['main'],
        light=color.get('light', None),
        lighter=color.get('lighter', None),
        dark=color.get('dark', None),
        darker=color.get('darker', None),
        contrastText=get_contrast_text(color['main'])
    )

def augment_text_color(options: Dict) -> PaletteColor:
    return TypeText(
        primary=options.get('primary', None),
        secondary=options.get('secondary', None),
        disabled=options.get('disabled', None),
    )

# Tạo hàm create_palette để khởi tạo một đối tượng Palette

def create_palette(palette: Dict) -> Palette:
    mode = palette.get('mode', 'light')
    contrastThreshold = palette.get('contrastThreshold', 3)
    tonalOffset = palette.get('tonalOffset', 0.2)

    primary = palette.get('primary', get_default_primary(mode))
    secondary = palette.get('secondary', get_default_secondary(mode))
    error = palette.get('error', get_default_error(mode))

    text = palette.get('text', TypeText(primary="#000000", secondary="#757575", disabled="#bdbdbd"))
    
    grey_colors = Color(
        _50="#FFFFFF",
        _100="#F9FAFB",
        _200="#F4F6F8",
        _300="#DFE3E8",
        _400="#C4CDD5",
        _500="#919EAB",
        _600="#637381",
        _700="#454F5B",
        _800="#212B36",
        _900="#161C24"
    )
    
    return Palette(
        common=CommonColors(),
        mode=mode,
        contrastThreshold=contrastThreshold,
        tonalOffset=tonalOffset,
        primary=augment_color({"color": primary}),
        secondary=augment_color({"color": secondary}),
        error=augment_color({"color": error}),
        warning=augment_color({"color": palette.get('warning', get_default_error(mode))}),
        info=augment_color({"color": palette.get('info', get_default_primary(mode))}),
        success=augment_color({"color": palette.get('success', get_default_primary(mode))}),
        grey=grey_colors,
        text=augment_text_color(palette.get('text')),
        divider="#e0e0e0",
        action=TypeAction(
            active="#000000",
            hover="rgba(0, 0, 0, 0.04)",
            hoverOpacity=0.08,
            selected="rgba(0, 0, 0, 0.08)",
            selectedOpacity=0.08,
            disabled="rgba(0, 0, 0, 0.26)",
            disabledBackground="rgba(0, 0, 0, 0.12)",
            disabledOpacity=0.38,
            focus="rgba(0, 0, 0, 0.12)",
            focusOpacity=0.12,
            activatedOpacity=0.48
        ),
        background=TypeBackground(
            default=palette["background"]["default"], 
            paper=palette["background"]["paper"],
            neutral=palette["background"]["neutral"],
            notched=palette["background"]["notched"],
            navigation=palette["background"]["navigation"],
            main=palette["background"]["main"],
            second=palette["background"]["second"],
            thirty=palette["background"]["thirty"],
            content=palette["background"]["content"]
        ),
        getContrastText=get_contrast_text,
        augmentColor=augment_color
    )

# # # Ví dụ khởi tạo palette
# my_palette = create_palette({
#     'mode': 'light',
#     'primary': {'main': '#1976d2'},
#     'secondary': {'main': '#dc004e'}
# })



import asyncio
from typing import Optional, List, Callable, Union

from PySide6.QtCore import QTimer
from PySide6.QtWidgets import QWidget, QScrollArea

from .anim_controller import AnimationController

from qtmui.hooks import State


class AnimManager:
    def __init__(
        self,
        **kwargs,
    ):

        self.initial = kwargs.get("initial", {})
        self.animate = kwargs.get("animate", {})
        self.exit = kwargs.get("exit", {})
        self.variants = kwargs.get("variants", {})
        self.src = kwargs.get("src", None)
        
        if isinstance(self.variants, State):
            self.variants.valueChanged.connect(self.on_variants_changed)
        
        self.children_widgets: List[QWidget] = []
        self.child_controllers: List[AnimationController] = []

        # mark whether we've scheduled children animations already
        self._children_scheduled = False
        self._connected = False

    def on_variants_changed(self, new_variants):
        if self.parent() and hasattr(self.parent(), "child_controllers"):
            for ctl in self.parent().child_controllers:
                ctl.variants = new_variants
                ctl._played = False

            if self.parent().children_widgets:
                QTimer.singleShot(0, self.parent()._children_animation)

    def _children_animation(self):
        self._schedule_children_animation_task = asyncio.ensure_future(self._schedule_children_animation())
        
    def _children_apply_initial(self, fn):
        self._schedule_children_apply_initial_task = asyncio.ensure_future(fn())


    def setup_child(self, child: QWidget = None):
        """Setup children widgets and their AnimationController controllers."""
        self.children_widgets.append(child)
        child_variants = child.getVariants() if hasattr(child, "getVariants") else {}
        child_ctrl = AnimationController(child, child_variants)
        self.child_controllers.append(child_ctrl)
        
    def getVariants(self) -> dict:
        """Get variant definition by name."""
        variant = {}
        if isinstance(self.variants, State):
            variant = self.variants.value
        elif isinstance(self.variants, dict):
            variant = self.variants
        return variant

    def find_scroll_area(self) -> Optional[QScrollArea]:
        p = self.parentWidget()
        while p:
            if isinstance(p, QScrollArea):  # Hoặc isinstance(p, Scrollbar) nếu Scrollbar kế thừa QScrollArea
                return p
            p = p.parentWidget()
        return None

    async def _schedule_children_animation(self):
        try:
            """Apply initial state for all children and then play them with stagger/delay from this Box variants."""
            # # Áp dụng initial cho tất cả children trước
            for ctrl in self.child_controllers:
                ctrl.apply_initial()

            # Kiểm tra nếu container nằm trong QScrollArea (Scrollbar giả sử là subclass của QScrollArea)
            scroll_area = self.find_scroll_area()
            
            if scroll_area:
                # Kết nối signal nếu chưa
                if not self._connected:
                    scroll_area.verticalScrollBar().valueChanged.connect(self._animate_visible_children)
                    self._connected = True
                # Trigger initial animation cho các child visible ban đầu, 100 để các child setStyleSheet xong và có height
                QTimer.singleShot(0, lambda: self._animate_visible_children(scroll_area.verticalScrollBar().value()))
            else:
                # Không có scroll, play tất cả với stagger như cũ
                animate_def = self.getVariants().get("animate", {}) or {}
                transition = animate_def.get("transition", {}) or {}
                base_delay = int(transition.get("delayChildren", 0) * 1000)
                stagger = int(transition.get("staggerChildren", 0) * 1000)
                for idx, ctrl in enumerate(self.child_controllers):
                    delay_ms = base_delay + idx * stagger
                    ctrl.play(delay_ms=delay_ms)
        except Exception as e:
            pass

    def _animate_visible_children(self, scroll_pos: int):
        scroll_area = self.find_scroll_area()
        if not scroll_area:
            return

        viewport_h = scroll_area.viewport().height()
        
        visible_to_play = []
        for idx, ctrl in enumerate(self.child_controllers):
            if ctrl._played:
                continue
            child = self.children_widgets[idx]
            child_y = child.pos().y()  # Vị trí y tương đối với container
            child_h = child.height()
            # Kiểm tra nếu child nằm trong vùng visible (có overlap với viewport)
            if (child_y < scroll_pos + viewport_h) and (child_y + child_h > scroll_pos):
                visible_to_play.append((idx, ctrl))

        if not visible_to_play:
            return

        # Sắp xếp theo thứ tự index để giữ order
        visible_to_play.sort(key=lambda x: x[0])

        # Lấy transition để áp dụng stagger cho batch visible này
        animate_def = self.getVariants().get("animate", {}) or {}
        transition = animate_def.get("transition", {}) or {}
        stagger = transition.get("staggerChildren", 0)
        stagger_ms = int(stagger * 1000) if stagger < 100 else int(stagger)
        # Không dùng delayChildren cho các batch sau, chỉ stagger nội bộ batch
        delay_ms = 0

        for _, ctrl in visible_to_play:
            ctrl.play(delay_ms=delay_ms)
            delay_ms += stagger_ms


    def _runAnimation(self):
        # We must schedule apply_initial and play AFTER layout has assigned positions.
        # Use singleShot(0) to run after the event loop processes layout.
        if self.children_widgets and not self._children_scheduled:
            QTimer.singleShot(0, self._children_animation)
            self._children_scheduled = True
        elif self.variants and not hasattr(self.parent(), "variants"): # kiểm tra rằng parrent có variants = {} => không nhận kiểm soát animation từ parent
            self.animCtl = AnimationController(self, self.getVariants())
            self.animCtl.apply_initial()
            self.animCtl.play(delay_ms=50)
        elif self.variants != {} and (hasattr(self.parent(), "variants") and self.parent().variants == {}): # kiểm tra rằng parrent có variants = {} => không nhận kiểm soát animation từ parent
            self.animCtl = AnimationController(self, self.getVariants())
            self.animCtl.apply_initial()
            QTimer.singleShot(0, self._animCtlPlayTask)
            
        # self._scaleX
            
            
    def _animCtlPlayTask(self):
        self._schedule__animCtlPlayTask = asyncio.ensure_future(self._animCtlPlay())
            
    async def _animCtlPlay(self):
        self.animCtl.play(delay_ms=0)
            
    def showEvent(self, event):
        QTimer.singleShot(0, self._runAnimation)


from qtmui.material.styles import useTheme
from PySide6.QtWidgets import QHBoxLayout, QFrame, QSizePolicy
from PySide6.QtCore import Qt
from ..styles.create_theme.components.get_qss_styles import get_qss_style

from ..py_svg_widget import PySvgWidget
from ..widget_base.widget_base import PyWidgetBase


class ListItemIcon(QFrame, PyWidgetBase):
    def __init__(self, 
                 children=None,
                 **kwargs
                ):
        super().__init__(**kwargs)
        # self.setObjectName("PyListItemIcon")
        self._setUpUi()

        # Gán các prop thành thuộc tính của class
        self.kwargs = kwargs

        self._children = children

        self._init_ui()

        self.theme = useTheme()
        self.useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self.destroyed.connect(self._on_destroyed)
        self._set_stylesheet()


    def _init_ui(self):

        # Layout cơ bản cho icon
        self._layout = QHBoxLayout(self)
        self._layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self._layout)
        self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        # self.setFixedWidth(24)
        self.setAttribute(Qt.WA_TransparentForMouseEvents, True) # Điều này cho phép các sự kiện chuột (bao gồm hover) có thể đi qua và được lắng nghe bởi QPushButton.
        self.theme = useTheme()

        # Thêm các children (nếu có)
        if self._children:
            if not isinstance(self._children, list):
                raise TypeError("children must be type (list)")
            for child in self._children:
                self.layout().addWidget(child)
        else:
            # self.layout().addWidget(PySvgWidget(color=self.theme.palette.primary.main, **self.kwargs))
            self.layout().addWidget(PySvgWidget(**self.kwargs))


    def _set_stylesheet(self, component_styled=None):
        self.theme = useTheme()
        self._update_component_styles(self.theme, component_styled)

        ownerState = {
            **self.kwargs
        }

        # print('________00000000000', self.styleFn(self.theme))

        PyListItemIcon_root = self.component_styles[f"PyListItemIcon"].get("styles")["root"](ownerState)
        PyListItemIcon_root_qss = get_qss_style(PyListItemIcon_root)


        self.setStyleSheet(
            f"""
                ListItemIcon {{
                    {PyListItemIcon_root_qss}
                }}
            """
        )
        
        
        



from PySide6.QtWidgets import QFrame, QWidget, QHBoxLayout
from PySide6.QtCore import QObject


class TableRow(QWidget):
    def __init__(
                self,
                children: object = None,
                data: object = None,
                hover: bool = False,
                selected: bool = False,
                ):
        super().__init__()

        self._children = children
        self._data = data

        self._init_ui()

    def _init_ui(self):
        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        if self._children:
            for widget in self._children:
                if widget is not None:
                    self.layout().addWidget(widget)

    def add_children(self, children: list):
        if children:
            for widget in children:
                if widget is not None:
                    self.layout().addWidget(widget)




from __future__ import annotations

from typing import Callable, Optional, Union, List, Dict
import math

from PySide6.QtCore import Qt, QPointF
from PySide6.QtWidgets import QWidget, QVBoxLayout
from PySide6.QtCharts import (
    QChart,
    QChartView,
    QLineSeries,
    QValueAxis,
    QCategoryAxis,
    QPolarChart,
)
from PySide6.QtGui import QPainter, QColor, QBrush, QPen, QFont

from qtmui.hooks import State
from qtmui.material.styles import useTheme


class ChartRadar(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        series: List[Dict] = [],
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        options: Optional[Dict] = None,
        key: str = None,
        title: Optional[Union[State, str, Callable]] = None,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)

        self.series = series
        self.options = options or {}
        self.title = title
        self._height = height
        self._width = width

        self._init_radar_chart()

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

    def _init_radar_chart(self):
        self.theme = useTheme()
        self.chart = QPolarChart()


        # Lấy categories từ options, nếu không có thì dùng list rỗng
        self.categories = self.options.get("xaxis", {}).get("categories", [])

        if self._width:
            self.setFixedWidth(self._width)
        if self._height:
            self.setFixedHeight(self._height)

        # Tạo các trục tọa độ cực
        self.angular_axis = self._create_angular_axis()
        self.radial_axis = self._create_radial_axis()

        # Thêm các trục vào chart
        self.chart.addAxis(self.angular_axis, QPolarChart.PolarOrientationAngular)
        self.chart.addAxis(self.radial_axis, QPolarChart.PolarOrientationRadial)

        # Thêm các series dữ liệu
        self._add_data_series()

        # Cấu hình chart
        if self.title:
            self.chart.setTitle(self.title)

        self.chart.legend().setAlignment(Qt.AlignBottom)
        self.chart.legend().show()

        # Tạo ChartView
        self.chart_view = QChartView(self.chart)
        self.chart_view.setRenderHint(QPainter.Antialiasing)
        self.layout().addWidget(self.chart_view)

    def _create_angular_axis(self):
        axis = QCategoryAxis()
        angle_step = 360 / len(self.categories) if self.categories else 0

        # Tạo nhãn theo thứ tự ban đầu:
        for i, category in enumerate(self.categories):
            angle = i * angle_step + 60  # Giá trị góc tính theo 0, 60, 120, ... nếu có 6 category
            axis.append(category, angle)

        axis.setRange(0, 360)
        axis.setLabelsFont(QFont("Arial", 10))
        return axis


    def _create_radial_axis(self):
        axis = QValueAxis()
        axis.setTickCount(5)
        axis.setLabelFormat("%d")
        axis.setRange(0, 100)  # Giả định giá trị max là 100
        return axis

    def _add_data_series(self):
        colors = self.options.get("colors", [
            self.theme.palette.primary.main,
            self.theme.palette.secondary.main,
            self.theme.palette.error.main,
        ])

        max_value = 100  # Giả định giá trị max là 100

        for idx, series_data in enumerate(self.series):
            series = QLineSeries()
            series.setName(series_data.get("name", f"Series {idx+1}"))

            angle_step = 360 / len(series_data["data"])
            for i, value in enumerate(series_data["data"]):
                # Tính góc: 0, 60, 120, ... (không cần cộng thêm offset)
                angle = i * angle_step
                radius = (value / max_value) * 100  # Scale về khoảng 0-100
                series.append(QPointF(angle, radius))

            pen = QPen(QColor(colors[idx % len(colors)]))
            pen.setWidth(2)
            series.setPen(pen)

            self.chart.addSeries(series)
            series.attachAxis(self.angular_axis)
            series.attachAxis(self.radial_axis)


    def _set_stylesheet(self):
        theme = useTheme()
        # Cấu hình màu sắc
        self.chart.setBackgroundBrush(QBrush(QColor(theme.palette.background.paper)))
        self.chart.setTitleBrush(QBrush(QColor(theme.palette.text.primary)))

        # Cập nhật màu cho các trục
        text_color = QColor(theme.palette.text.secondary)
        for axis in self.chart.axes():
            axis.setLabelsBrush(QBrush(text_color))
            axis.setLinePenColor(text_color)
            axis.setGridLineColor(QColor(theme.palette.divider))

        # Cập nhật legend
        self.chart.legend().setLabelColor(text_color)



from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from ...theme_reducer import ThemeState

from ..properties_name import *
from .....system.color_manipulator import alpha

def py_filled_input(_theme):
    theme: ThemeState = _theme

    _color = {
        'focused': theme.palette.text.primary,
        'active': theme.palette.text.secondary,
        'placeholder': theme.palette.text.disabled,
    }

    return {
        'MuiFilledInput': {
            'styles': {
                'root': {
                    backgroundColor: alpha(theme.palette.grey._500, 0.16),
                    border: f"1px solid transparent",
                    "slots": {
                        "hover": {
                            backgroundColor: alpha(theme.palette.grey._500, 0.20),
                        },
                        "focus": {
                            backgroundColor: alpha(theme.palette.grey._500, 0.24),
                        },
                        'error': {
                            borderColor: theme.palette.error.main
                        },
                        'disabled': {
                            borderColor: theme.palette.action.disabledBackground
                        },
                    },
                    "props": {
                        "error": {
                            borderColor: theme.palette.error.main
                        },
                        "validate": {
                            borderColor: theme.palette.success.main
                        },

                    },
                },
                "title": {
                    top: 6,
                    left: 8,
                    "props": {
                        "hasStartAdornment": {
                            left: 30,
                        },
                    }
                },
                "inputField": {
                    "root": {
                        paddingLeft: 8,
                        "props": {
                            "hasValue": {
                                "small": {
                                    paddingTop: 12,
                                },
                                "medium": {
                                    paddingTop: 16,
                                },
                            },
                            "hasStartAdornment": {
                                paddingLeft: 0,
                            },
                        }
                    }
                }
            },
        },
    }


from typing import Optional, Union, Dict, Callable, List
from PySide6.QtWidgets import QLabel, QWidget, QHBoxLayout, QFrame
from PySide6.QtCore import Qt
import uuid
from qtmui.hooks import State, useEffect
from ...material.styles import useTheme
from qtmui.i18n.use_translation import translate, i18n
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..utils.validate_params import _validate_param
from qtmui.utils.translator import getTranslatedText

class FormHelperText(QLabel):
    """
    A component that provides helper text for form controls.

    The `FormHelperText` component is used to display helper or error messages below
    form controls, supporting all props of the Material-UI `FormHelperText` component.

    Parameters
    ----------
    children : State, str, QWidget, Callable, or None, optional
        The content of the component. If ' ', reserves one line height for a future message.
        Can be a string, a QWidget, a translation function, or a `State` object for dynamic updates.
        Default is None.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State or str, optional
        The component used for the root node (e.g., "p"). Default is None (uses QLabel or QFrame).
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, the helper text is displayed in a disabled state. Default is False.
        Can be a `State` object for dynamic updates.
    error : State or bool, optional
        If True, the helper text is displayed in an error state. Default is False.
        Can be a `State` object for dynamic updates.
    filled : State or bool, optional
        If True, the helper text uses filled classes key. Default is False.
        Can be a `State` object for dynamic updates.
    focused : State or bool, optional
        If True, the helper text uses focused classes key. Default is False.
        Can be a `State` object for dynamic updates.
    margin : State or str, optional
        If "dense", adjusts vertical spacing (normally from FormControl). Default is "dense".
        Can be a `State` object for dynamic updates.
    required : State or bool, optional
        If True, the helper text uses required classes key. Default is False.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    variant : State or str, optional
        The variant to use ("filled", "outlined", "standard", or custom string). Default is "standard".
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QLabel` or `QFrame` class,
        supporting props of the native component (e.g., parent, style, className).

    Attributes
    ----------
    VALID_MARGINS : list[str]
        Valid values for `margin`: ["dense"].
    VALID_VARIANTS : list[str]
        Valid values for `variant`: ["filled", "outlined", "standard"].

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `parent`, `style`, `className`).
    - If `children` is a `QWidget`, the component uses a `QFrame` with a `QHBoxLayout` to contain it.
    - If `children` is ' ', the component reserves one line height for a future message.

    Demos:
    - FormHelperText: https://qtmui.com/material-ui/qtmui-formhelpertext/

    API Reference:
    - FormHelperText API: https://qtmui.com/material-ui/api/form-helper-text/
    """

    VALID_MARGINS = ["dense"]
    VALID_VARIANTS = ["filled", "outlined", "standard"]

    def __init__(
        self,
        children: Optional[Union[State, str, QWidget, Callable]] = None,
        classes: Optional[Union[State, Dict]] = None,
        component: Optional[Union[State, str]] = None,
        disabled: Union[State, bool] = False,
        error: Union[State, bool] = False,
        filled: Union[State, bool] = False,
        focused: Union[State, bool] = False,
        margin: Union[State, str] = "dense",
        required: Union[State, bool] = False,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        variant: Union[State, str] = "standard",
        **kwargs
    ):
        super().__init__(**kwargs)
        self.setObjectName(f"MuiFormHelperText-{str(uuid.uuid4())}")

        # Initialize theme
        self.theme = useTheme()

        # Store widget references to prevent Qt deletion
        self._widget_references = []

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_disabled(disabled)
        self._set_error(error)
        self._set_filled(filled)
        self._set_focused(focused)
        self._set_margin(margin)
        self._set_required(required)
        self._set_sx(sx)
        self._set_variant(variant)

        # Setup UI
        self._init_ui()

        # Apply styles
        self._set_stylesheet()

        # Connect signals
        self._connect_signals()
        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        i18n.langChanged.connect(self.retranslateUi)
        self.retranslateUi()

    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="children", supported_signatures=Union[State, str, QWidget, Callable, type(None)])
    def _set_children(self, value):
        """Assign value to children."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="component", supported_signatures=Union[State, str, type(None)])
    def _set_component(self, value):
        """Assign value to component."""
        self._component = value

    def _get_component(self):
        """Get the component value."""
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="disabled", supported_signatures=Union[State, bool])
    def _set_disabled(self, value):
        """Assign value to disabled."""
        self._disabled = value
        self.setEnabled(not self._get_disabled())

    def _get_disabled(self):
        """Get the disabled value."""
        return self._disabled.value if isinstance(self._disabled, State) else self._disabled

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="error", supported_signatures=Union[State, bool])
    def _set_error(self, value):
        """Assign value to error."""
        self._error = value

    def _get_error(self):
        """Get the error value."""
        return self._error.value if isinstance(self._error, State) else self._error

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="filled", supported_signatures=Union[State, bool])
    def _set_filled(self, value):
        """Assign value to filled."""
        self._filled = value

    def _get_filled(self):
        """Get the filled value."""
        return self._filled.value if isinstance(self._filled, State) else self._filled

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="focused", supported_signatures=Union[State, bool])
    def _set_focused(self, value):
        """Assign value to focused."""
        self._focused = value

    def _get_focused(self):
        """Get the focused value."""
        return self._focused.value if isinstance(self._focused, State) else self._focused

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="margin", supported_signatures=Union[State, str])
    def _set_margin(self, value):
        """Assign value to margin."""
        if value not in self.VALID_MARGINS:
            raise ValueError(f"margin must be one of {self.VALID_MARGINS}, got {value}")
        self._margin = value

    def _get_margin(self):
        """Get the margin value."""
        return self._margin.value if isinstance(self._margin, State) else self._margin

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="required", supported_signatures=Union[State, bool])
    def _set_required(self, value):
        """Assign value to required."""
        self._required = value

    def _get_required(self):
        """Get the required value."""
        return self._required.value if isinstance(self._required, State) else self._required

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.formhelpertext", param_name="variant", supported_signatures=Union[State, str])
    def _set_variant(self, value):
        """Assign value to variant."""
        if value not in self.VALID_VARIANTS and not isinstance(value, str):
            raise ValueError(f"variant must be one of {self.VALID_VARIANTS} or a custom string, got {value}")
        self._variant = value

    def _get_variant(self):
        """Get the variant value."""
        return self._variant.value if isinstance(self._variant, State) else self._variant

    def _init_ui(self):
        """Initialize the UI based on props."""
        children = self._get_children()

        # If children is a QWidget, use QFrame with QHBoxLayout
        if isinstance(children, QWidget):
            self.__class__ = QFrame  # Change base class to QFrame
            self.setLayout(QHBoxLayout())
            self.layout().setContentsMargins(0, 0, 0, 0)
            self.layout().addWidget(children)
            self._widget_references.append(children)
        else:
            if self.layout():
                self.setLayout(None)  # Remove layout if previously set
            if isinstance(children, str):
                self.setText(getTranslatedText(self._children))
                
            # self.setEnabled(not self._get_disabled())

        # Apply margin
        if self._get_margin() == "dense":
            self.setContentsMargins(4, 2, 4, 2)  # MUI dense margin

    def retranslateUi(self):
        """Update the UI text based on the current language and children."""
        children = self._get_children()
        if isinstance(children, str):
            self.setText(getTranslatedText(self._children))

    def _set_stylesheet(self, component_styled=None):
        """Apply styles based on theme, classes, and sx."""
        self.theme = useTheme()
        component_styled = component_styled or self.theme.components
        form_helper_text_styles = component_styled.get("FormHelperText", {}).get("styles", {})
        root_styles = form_helper_text_styles.get("root", {})
        root_qss = get_qss_style(root_styles)

        # Handle variant
        variant_styles = form_helper_text_styles.get("props", {}).get(f"{self._get_variant()}Variant", {})
        variant_qss = get_qss_style(variant_styles)

        # Handle color based on error and disabled
        color = (
            self.theme.palette.text.disabled if self._get_disabled()
            else self.theme.palette.error.main if self._get_error()
            else self.theme.palette.text.secondary
        )
        color_qss = f"color: {color};"

        # Handle sx
        sx = self._get_sx()
        sx_qss = ""
        if sx:
            if isinstance(sx, (list, dict)):
                sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
            elif isinstance(sx, Callable):
                sx_result = sx()
                if isinstance(sx_result, (list, dict)):
                    sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
                elif isinstance(sx_result, str):
                    sx_qss = sx_result
            elif isinstance(sx, str) and sx != "":
                sx_qss = sx

        # Handle classes
        classes = self._get_classes()
        classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}") if classes else ""

        # Apply MUI classes
        mui_classes = ["MuiFormHelperText-root", f"MuiFormHelperText-{self._get_variant()}"]
        if self._get_disabled():
            mui_classes.append("Mui-disabled")
        if self._get_error():
            mui_classes.append("Mui-error")
        if self._get_filled():
            mui_classes.append("MuiFormHelperText-filled")
        if self._get_focused():
            mui_classes.append("MuiFormHelperText-focused")
        if self._get_required():
            mui_classes.append("MuiFormHelperText-required")

        stylesheet = f"""
            #{self.objectName()} {{
                {root_qss}
                {variant_qss}
                {color_qss}
                {classes_qss}
            }}
            {sx_qss}
        """
        self.setStyleSheet(stylesheet)

    def _connect_signals(self):
        """Connect valueChanged signals of State parameters to their slots."""
        if isinstance(self._children, State):
            self._children.valueChanged.connect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.connect(self._on_classes_changed)
        if isinstance(self._component, State):
            self._component.valueChanged.connect(self._on_component_changed)
        if isinstance(self._disabled, State):
            self._disabled.valueChanged.connect(self._on_disabled_changed)
        if isinstance(self._error, State):
            self._error.valueChanged.connect(self._on_error_changed)
        if isinstance(self._filled, State):
            self._filled.valueChanged.connect(self._on_filled_changed)
        if isinstance(self._focused, State):
            self._focused.valueChanged.connect(self._on_focused_changed)
        if isinstance(self._margin, State):
            self._margin.valueChanged.connect(self._on_margin_changed)
        if isinstance(self._required, State):
            self._required.valueChanged.connect(self._on_required_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.connect(self._on_sx_changed)
        if isinstance(self._variant, State):
            self._variant.valueChanged.connect(self._on_variant_changed)

    def _on_children_changed(self):
        """Handle changes to children."""
        self._set_children(self._children)
        self._init_ui()
        self._set_stylesheet()

    def _on_classes_changed(self):
        """Handle changes to classes."""
        self._set_classes(self._classes)
        self._set_stylesheet()

    def _on_component_changed(self):
        """Handle changes to component."""
        self._set_component(self._component)
        self._init_ui()
        self._set_stylesheet()

    def _on_disabled_changed(self):
        """Handle changes to disabled."""
        self._set_disabled(self._disabled)
        self._set_stylesheet()

    def _on_error_changed(self):
        """Handle changes to error."""
        self._set_error(self._error)
        self._set_stylesheet()

    def _on_filled_changed(self):
        """Handle changes to filled."""
        self._set_filled(self._filled)
        self._set_stylesheet()

    def _on_focused_changed(self):
        """Handle changes to focused."""
        self._set_focused(self._focused)
        self._set_stylesheet()

    def _on_margin_changed(self):
        """Handle changes to margin."""
        self._set_margin(self._margin)
        self._init_ui()

    def _on_required_changed(self):
        """Handle changes to required."""
        self._set_required(self._required)
        self._set_stylesheet()

    def _on_sx_changed(self):
        """Handle changes to sx."""
        self._set_sx(self._sx)
        self._set_stylesheet()

    def _on_variant_changed(self):
        """Handle changes to variant."""
        self._set_variant(self._variant)
        self._set_stylesheet()

 


# testCollections.py
#
# This code tests both the client and server side of collections
# and enumerators.
#
# Also has the side effect of testing some of the PythonCOM error semantics.
import sys
import unittest

import pythoncom
import win32com.client
import win32com.server.util
import win32com.test.util
import winerror


def MakeEmptyEnum():
    # create the Python enumerator object as a real COM object
    o = win32com.server.util.wrap(win32com.server.util.Collection())
    return win32com.client.Dispatch(o)


def MakeTestEnum():
    # create a sub-collection, just to make sure it works :-)
    sub = win32com.server.util.wrap(
        win32com.server.util.Collection(["Sub1", 2, "Sub3"])
    )
    # create the Python enumerator object as a real COM object
    o = win32com.server.util.wrap(win32com.server.util.Collection([1, "Two", 3, sub]))
    return win32com.client.Dispatch(o)


def TestEnumAgainst(o, check):
    for i in range(len(check)):
        assert (
            o(i) == check[i]
        ), f"Using default method gave the incorrect value - {o(i)!r}/{check[i]!r}"

    for i in range(len(check)):
        assert (
            o.Item(i) == check[i]
        ), f"Using Item method gave the incorrect value - {o(i)!r}/{check[i]!r}"

    # First try looping.
    cmp = []
    for s in o:
        cmp.append(s)

    assert (
        cmp[: len(check)] == check
    ), f"Result after looping isn't correct - {cmp[: len(check)]!r}/{check!r}"

    for i in range(len(check)):
        assert o[i] == check[i], "Using indexing gave the incorrect value"


def TestEnum(quiet=None):
    if quiet is None:
        quiet = not "-v" in sys.argv
    if not quiet:
        print("Simple enum test")
    o = MakeTestEnum()
    check = [1, "Two", 3]
    TestEnumAgainst(o, check)

    if not quiet:
        print("sub-collection test")
    sub = o[3]
    TestEnumAgainst(sub, ["Sub1", 2, "Sub3"])

    # Remove the sublist for this test!
    o.Remove(o.Count() - 1)

    if not quiet:
        print("Remove item test")
    del check[1]
    o.Remove(1)
    TestEnumAgainst(o, check)

    if not quiet:
        print("Add item test")
    o.Add("New Item")
    check.append("New Item")
    TestEnumAgainst(o, check)

    if not quiet:
        print("Insert item test")
    o.Insert(2, -1)
    check.insert(2, -1)
    TestEnumAgainst(o, check)

    ### This does not work!
    # if not quiet: print("Indexed replace item test")
    # o[2] = 'Replaced Item'
    # check[2] = 'Replaced Item'
    # TestEnumAgainst(o, check)

    try:
        o()
        raise AssertionError(
            "default method with no args worked when it shouldn't have!"
        )
    except pythoncom.com_error as exc:
        assert (
            exc.hresult == winerror.DISP_E_BADPARAMCOUNT
        ), f"Expected DISP_E_BADPARAMCOUNT - got {exc}"

    try:
        o.Insert("foo", 2)
        raise AssertionError("Insert worked when it shouldn't have!")
    except pythoncom.com_error as exc:
        assert (
            exc.hresult == winerror.DISP_E_TYPEMISMATCH
        ), f"Expected DISP_E_TYPEMISMATCH - got {exc}"

    # Remove the sublist for this test!
    try:
        o.Remove(o.Count())
        raise AssertionError("Remove worked when it shouldn't have!")
    except pythoncom.com_error as exc:
        assert (
            exc.hresult == winerror.DISP_E_BADINDEX
        ), f"Expected DISP_E_BADINDEX - got {exc}"

    # Test an empty collection
    if not quiet:
        print("Empty collection test")
    o = MakeEmptyEnum()
    for item in o:
        raise AssertionError("Empty list performed an iteration")

    try:
        ob = o[1]
        raise AssertionError("Empty list could be indexed")
    except IndexError:
        pass

    try:
        ob = o[0]
        raise AssertionError("Empty list could be indexed")
    except IndexError:
        pass

    try:
        ob = o(0)
        raise AssertionError("Empty list could be indexed")
    except pythoncom.com_error as exc:
        assert (
            exc.hresult == winerror.DISP_E_BADINDEX
        ), f"Expected DISP_E_BADINDEX - got {exc}"


class TestCase(win32com.test.util.TestCase):
    def testEnum(self):
        TestEnum()


if __name__ == "__main__":
    unittest.main()



from typing import Dict, TYPE_CHECKING

from .properties_name import *

if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState

def dialog(_theme) -> Dict:
    theme: ThemeState = _theme
    # theme = _theme
    return {
        'PyDialog': {
            'styles': {
                'paper': {
                    "root": {
                        boxShadow: theme.customShadows.dialog,
                        
                        borderRadius: theme.shape.borderRadius * 2,
                        backgroundColor: theme.palette.background.paper,
                        'slots': {
                            'fullScreen': {
                                m: theme.spacing(2),
                            }
                        }
                    }
                },
                'paperFullScreen': {
                    borderRadius: 0,
                },
            },
        },
        'PyDialogTitle': {
            'styles': {
                'root': {
                    p: theme.spacing(3),
                },
            },
        },
        'PyDialogContent': {
            'styles': {
                'root': {
                    p: theme.spacing(0, 3),
                },
                'dividers': {
                    borderTop: 0,
                    borderBottomStyle: 'dashed',
                    paddingBottom: theme.spacing(3),
                },
            },
        },
        'PyDialogActions': {
            'styles': {
                'root': {
                    p: theme.spacing(3),
                    '& > :not(:first-of-type)': {
                        marginLeft: theme.spacing(1.5),
                    },
                },
            },
        },
    }



from typing import Optional, Union, Callable
from PySide6.QtWidgets import QSizePolicy, QHBoxLayout, QFrame
from PySide6.QtCore import Signal

from qtmui.hooks import State
from qtmui.common.ui_functions import clear_layout

from ..button.button import Button
from ..button.loading_button import LoadingButton
from ..qss_name import *

class SubmitButton(QFrame):
    clicked = Signal()
    def __init__(
            self,
            type: str = "submit",
            loading: Optional[State] = None,
            **kwargs
            ):
        super().__init__()
        self._kwargs = kwargs
        self._type = type
        self._loading = loading
        self._text = self._kwargs.get("text")
        self._onClick = self._kwargs.pop("onClick") if "onClick" in self._kwargs else None
        
        self._initUi()
        
        
    def _initUi(self):

        self._layout = QHBoxLayout()
        self.setLayout(self._layout)
        self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        self.layout().setContentsMargins(0,0,0,0)
        self._loadingButton = LoadingButton(loading=True, onClick=self.__onClick, **self._kwargs)
        # self._kwargs.pop("variant")
        self._button = Button(onClick=self.__onClick, **self._kwargs)
        self.layout().addWidget(self._loadingButton)
        self.layout().addWidget(self._button)
        
        if isinstance(self._loading, State):
            self._loading.valueChanged.connect(self._updateLoading)
            
        self._updateLoading()

    def _updateLoading(self, value=None):
        if isinstance(self._loading, State):
            if self._loading.value:
                self._loadingButton.setVisible(True)
                self._button.setVisible(False)
            else:
                self._loadingButton.setVisible(False)
                self._button.setVisible(True)
                
    def __onClick(self):
        self.clicked.emit()
        if isinstance(self._onClick, Callable):
            self._onClick()



# qtmui/material/chart/chart_bar.py
from __future__ import annotations

from typing import Any, Dict, List, Optional, Union
from PySide6.QtCore import Qt, QPointF, QRectF
from PySide6.QtWidgets import QWidget, QVBoxLayout, QGraphicsItem, QGraphicsSceneHoverEvent
from PySide6.QtCharts import (
    QChart, QChartView, QBarSeries, QPercentBarSeries,
    QBarSet, QBarCategoryAxis, QValueAxis
)
from PySide6.QtGui import (
    QPainter, QColor, QPen, QBrush, QFont, QFontMetrics, QLinearGradient, QGradient
)

from qtmui.material.styles import useTheme


class TooltipItem(QGraphicsItem):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.lines = []
        self.padding = 10
        self.bg_color = QColor(20, 20, 20, 240)
        self.border_color = QColor(255, 255, 255, 80)
        self.border_radius = 8
        self.font = QFont("Segoe UI", 9)
        self._w = self._h = 0
        self.setZValue(99999)
        self.hide()

    def clear(self):
        self.lines = []
        self._w = self._h = 0
        self.hide()

    def add_line(self, text: str, color: QColor):
        self.lines.append((text, color))
        fm = QFontMetrics(self.font)
        w = fm.horizontalAdvance(text) + 20
        h = fm.height() + 4
        self._w = max(self._w, w)
        self._h += h

    def boundingRect(self) -> QRectF:
        return QRectF(0, 0, self._w + self.padding * 2, self._h + self.padding * 2)

    def paint(self, painter: QPainter, *args):
        if not self.lines:
            return
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setFont(self.font)
        fm = QFontMetrics(self.font)

        rect = self.boundingRect()
        painter.setBrush(QBrush(self.bg_color))
        painter.setPen(QPen(self.border_color, 1))
        painter.drawRoundedRect(rect, self.border_radius, self.border_radius)

        y = self.padding
        for text, color in self.lines:
            painter.setBrush(QBrush(color))
            painter.setPen(Qt.NoPen)
            painter.drawEllipse(self.padding, y + 2, 8, 8)
            painter.setPen(QPen(QColor("white")))
            painter.drawText(self.padding + 16, y + fm.ascent() + 2, text)
            y += fm.height() + 4


class HoverChartView(QChartView):
    def __init__(self, chart, parent=None):
        super().__init__(chart, parent)
        self.owner: Optional['ChartBar'] = None
        self.setMouseTracking(True)

    def leaveEvent(self, event):
        if self.owner:
            self.owner.hide_tooltip()
        super().leaveEvent(event)


class ChartBar(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        type: str = "bar",
        series: Optional[Union[List[Dict[str, Any]], List[float], List[int]]] = None,
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        options: Optional[Dict[str, Any]] = None,
        key: Optional[str] = None,
        *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)

        self._type = type.lower()
        self._raw_series = series or []
        self._options = options or {}
        self._width = width
        self._height = height

        self._series = self._normalize_series(self._raw_series)
        self.current_hover_index = -1

        self._init_chart()

        theme = useTheme()
        theme.state.valueChanged.connect(self._apply_theme)
        self._apply_theme()

    def _normalize_series(self, raw) -> List[Dict[str, Any]]:
        if not raw:
            return []
        if isinstance(raw, (list, tuple)) and all(isinstance(x, (int, float)) for x in raw):
            return [{"name": "Series 1", "data": list(raw)}]
        if isinstance(raw, (list, tuple)) and all(isinstance(x, dict) for x in raw):
            return [{"name": x.get("name", f"Series {i+1}"), "data": x.get("data", [])} for i, x in enumerate(raw)]
        raise ValueError("series must be List[int/float] or List[dict(name=..., data=...)]")

    def _init_chart(self):
        if self._height: self.setFixedHeight(self._height)
        if self._width:  self.setFixedWidth(self._width)

        self.bar_series = QPercentBarSeries() if self._type == "barpercent" else QBarSeries()

        self.bar_sets = []
        self.colors = []
        palette_names = ["primary", "secondary", "success", "warning", "error", "info"]

        for idx, item in enumerate(self._series):
            name = item.get("name", f"Series {idx + 1}")
            data = [float(v) for v in item.get("data", [])]

            bar_set = QBarSet(name)
            bar_set.append(data)

            color_key = palette_names[idx % len(palette_names)]
            color = getattr(useTheme().palette, color_key).main
            qcolor = QColor(color)
            self.colors.append(qcolor)

            bar_set.setBrush(QBrush(qcolor))
            pen = QPen(QColor("#ffffff"))
            pen.setWidth(2)
            bar_set.setPen(pen)

            # Chỉ cần bắt hover vào, không cần xử lý status=False nữa
            bar_set.hovered.connect(self._on_bar_hovered)

            self.bar_sets.append(bar_set)
            self.bar_series.append(bar_set)

        self.chart = QChart()
        self.chart.addSeries(self.bar_series)
        self.chart.legend().setVisible(True)
        self.chart.legend().setAlignment(Qt.AlignTop)
        self.chart.setAnimationOptions(QChart.NoAnimation)
        self.chart.setBackgroundRoundness(12)

        self.categories = self._options.get("xaxis", {}).get("categories") or self._options.get("categories", [])

        self.axis_x = QBarCategoryAxis()
        if self.categories:
            self.axis_x.append(self.categories)
        self.chart.addAxis(self.axis_x, Qt.AlignBottom)
        self.bar_series.attachAxis(self.axis_x)

        self.axis_y = QValueAxis()
        self.axis_y.setLabelFormat("%.0f")
        self.chart.addAxis(self.axis_y, Qt.AlignLeft)
        self.bar_series.attachAxis(self.axis_y)

        self.tooltip = TooltipItem(self.chart)
        self.tooltip.hide()

        # Dùng custom view để bắt leaveEvent
        self.chart_view = HoverChartView(self.chart)
        self.chart_view.owner = self
        self.chart_view.setRenderHint(QPainter.Antialiasing)
        self.chart_view.scene().addItem(self.tooltip)
        self.layout().addWidget(self.chart_view)

    def _on_bar_hovered(self, status: bool, index: int):
        if not status:
            return  # Bỏ qua False – sẽ xử lý bằng leaveEvent

        self.current_hover_index = index

        # Tâm nhóm cột chính xác
        group_center_x = index + 0.5
        point_in_value = QPointF(group_center_x, self.axis_y.max() * 0.1)
        center_pos = self.chart.mapToPosition(point_in_value, self.bar_series)

        # Tooltip content
        self.tooltip.clear()
        title = self.categories[index] if self.categories and index < len(self.categories) else f"Col {index}"
        self.tooltip.add_line(title, QColor("#cccccc"))

        for i, bar_set in enumerate(self.bar_sets):
            value = bar_set.at(index) if index < bar_set.count() else 0
            self.tooltip.add_line(f"{bar_set.label()}: {value:,.0f}", self.colors[i])

        # Căn giữa tooltip
        rect = self.tooltip.boundingRect()
        plot_area = self.chart.plotArea()

        tx = center_pos.x() - rect.width() / 2
        tx = max(plot_area.left() + 10, min(tx, plot_area.right() - rect.width() - 10))
        ty = plot_area.top() + 10

        self.tooltip.setPos(tx, ty)
        self.tooltip.show()

    def hide_tooltip(self):
        """Gọi khi chuột rời khỏi toàn bộ chart"""
        self.current_hover_index = -1
        self.tooltip.clear()

    def _apply_theme(self):
        theme = useTheme()

        grad = QLinearGradient(0, 0, 0, 1)
        grad.setCoordinateMode(QGradient.ObjectBoundingMode)
        grad.setColorAt(0.0, QColor(theme.palette.background.paper))
        grad.setColorAt(1.0, QColor(theme.palette.background.main))
        self.chart.setBackgroundBrush(QBrush(grad))

        text_color = QColor(theme.palette.text.secondary)
        divider = QColor(theme.palette.divider)

        self.chart.setTitleBrush(text_color)
        for axis in self.chart.axes():
            axis.setLabelsBrush(text_color)
            axis.setLinePenColor(divider)
            axis.setGridLineColor(divider)
            axis.setGridLinePen(QPen(divider, 0, Qt.DashLine))

        legend = self.chart.legend()
        legend.setLabelBrush(text_color)

        palette_names = ["primary", "secondary", "success", "warning", "error", "info"]
        for i, bar_set in enumerate(self.bar_sets):
            color_key = palette_names[i % len(palette_names)]
            color = getattr(theme.palette, color_key).main
            bar_set.setBrush(QBrush(QColor(color)))
            self.colors[i] = QColor(color)

        self.chart_view.update()


# coding: utf-8
import asyncio
import uuid
from typing import  Union, Optional, Dict

from PySide6.QtWidgets import (
    QHeaderView, 
    QWidget, 
    QTableView, 
    QWidget, 
    QHBoxLayout,
)
from PySide6.QtCore import (
    Qt, 
    QModelIndex, 
    Property, 
    Signal,
    QAbstractTableModel,
    QTimer,
    QItemSelection,
)

from typing import TYPE_CHECKING, Callable


from qtmui.utils.translator import getTranslatedText

if TYPE_CHECKING:
    from .table_cell import TableViewCell

from qtmui.hooks import State, useEffect

from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from qtmui.material.styles import useTheme
from ....i18n.use_translation import i18n

from qtmui.material.table.table_head import TableHead

from ...checkbox import Checkbox

from .table_base import TableBase

INDEXS = []
SCROLL_POS = None


class AbstractTableModel(QAbstractTableModel):
    def headerData(self, section: int, orientation: Qt.Orientation, role: int):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return getTranslatedText(self._headerLabels[section])
        if orientation == Qt.Vertical and role == Qt.DisplayRole:
            return f"{section + 1}"
        
        if role == Qt.TextAlignmentRole:
            # Cài đặt căn chỉnh cho từng cột
            if self._headLabel[section].get("align") == "center":
                return int(Qt.AlignCenter | Qt.AlignVCenter)
            elif self._headLabel[section].get("align") == "right":
                return int(Qt.AlignRight | Qt.AlignVCenter)
            else:  # left
                return int(Qt.AlignLeft | Qt.AlignVCenter)

    def __init__(self, 
                parent=None, 
                headLabel=None, 
                data=None
                ):
        
        self._headLabel = headLabel
        self._headerLabels = [item.get('label') if item.get('label') is not None else ""  for item in self._headLabel]

        super(AbstractTableModel, self).__init__(parent)

        self._data = data

    def rowCount(self, n=None):
        if isinstance(self._data, list):
            return len(self._data)
        else:
            return 0

    def columnCount(self, n=None):
        return len(self._headLabel)

    def flags(self, index: QModelIndex) -> Qt.ItemFlags:
        return Qt.ItemIsDropEnabled | Qt.ItemIsEnabled | Qt.ItemIsEditable | Qt.ItemIsSelectable | Qt.ItemIsDragEnabled

    def data(self, index, role=Qt.ForegroundRole):
        if index.isValid():
            if role == Qt.ItemDataRole.DisplayRole or role == Qt.EditRole:
                try:
                    widget_info = self._data[index.row()][index.column()]
                    return widget_info
                except Exception as e:
                    print('eeeeeeeeeee', e)
                    return ""
            # if role == Qt.ItemDataRole.DisplayRole or role == Qt.EditRole:
            #     if index.column() == 1 or index.column() == 13:
            #         return # khong set text content cho column co widget
            #     try:
            #         value = str(self._data[index.row()][index.column()])
            #     except:
            #         value = None
            #     pass
            #         return value or ""

class TableView(TableBase, QTableView):
    """ Table view """
    setIndexWidgetSignal = Signal(object)
    removeIndexWidgetSignal = Signal(QModelIndex)

    def __init__(
                self, 
                parent=None,
                fullWidth: bool = True,
                isBorderVisible: bool = False,
                tableHead: TableHead = None,
                children: list = None,
                loading: Optional[Union[bool, State]] = None,
                sortingEnabled: bool = False,
                size: str= None,
                model: object= None,
                dragEnable: Optional[bool]= False,
                dragDropMode: Optional[QTableView.DragDropMode] = None,
                selectionMode: Optional[QTableView.SelectionMode] = None,
                selectionBehavior: Optional[QTableView.SelectionBehavior]= None,
                onSelectionChanged: Optional[Callable]= None,
                maxHeight: Optional[int] = None,
                rowHeight: int= 40,
                sx: Optional[Union[dict, State]] = None,
                ):
        super().__init__(parent)
        self._children: list = children
        self._loading = loading
        self._fullWidth = fullWidth
        self._tableHead = tableHead
        self._isBorderVisible = isBorderVisible
        self._sortingEnabled = sortingEnabled
        self._rowHeight = rowHeight
        self._sx = sx
        self._model: State = model
        self._maxHeight: int = maxHeight
        
        self._dragEnable = dragEnable
        self._selectionMode = selectionMode
        self._selectionBehavior = selectionBehavior
        self._onSelectionChanged = onSelectionChanged
        self._dragDropMode = dragDropMode

        self._header_items = []

        self._last_visible_indexes = []

        # if self._tableHead and isinstance(self._tableHead._numSelected, State):
        #     self._selected, self._setSelected = useMemo(lambda: self._tableHead._numSelected.value, [self._tableHead._numSelected])
        # else:
        #     self._selected, self._setSelected = useState(0)
            

        self._init_ui()

    def _init_ui(self):
        self.setObjectName(str(uuid.uuid4()))
        # self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)
        
        if self._dragEnable:
            self.setDragEnabled(True)
            if self._selectionMode:
                self.setSelectionMode(self._selectionMode)
            if self._selectionBehavior:
                self.setSelectionBehavior(self._selectionBehavior)
            if self._dragDropMode:
                self.setDragDropMode(self._dragDropMode)


        self.verticalHeader().hide()
        self.setViewportMargins(0, 0, 0, 0)
        self.setMinimumHeight(50)

        if self._sortingEnabled:
            self.setSortingEnabled(True)

        self.theme = useTheme()


        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        self._set_stylesheet()


        # table view from gridview
        self.model_data = self._model.value


        # for i in range(len(self.model_data[0])):
        #     # self.setItemDelegateForColumn(i, StyledItem(self))
        #     self.setItemDelegateForColumn(i, TableItemDelegate(self))

        i18n.langChanged.connect(self.__setup_header)

        self.setIndexWidgetSignal.connect(self._set_index_widget)
        self.removeIndexWidgetSignal.connect(self._remove_index_widget)

        self._model.valueChanged.connect(self._set_model_data)
        self._set_model_data(self.model_data)

        scrollbar = self.verticalScrollBar()
        scrollbar.sliderReleased.connect(self.onScrollReleased)

        self.__setup_header()
        
  
    def _onSelectionChange(self, selected: QItemSelection, deselected: QItemSelection):
        """Được gọi khi người dùng thay đổi dòng hoặc ô được chọn"""
        indexes = selected.indexes()
        if indexes:
            row = indexes[0].row()
            # print(f"✅ Dòng được chọn: {self.getselectedRowsFast()}", self._tableHead._tableSelectedAction.isVisible())
            if self._tableHead._tableSelectedAction and not self._tableHead._tableSelectedAction.isVisible():
                self._tableHead._tableSelectedAction.setVisible(True)
            
        else:
            # print("❌ Không có dòng nào được chọn")
            if self._tableHead._tableSelectedAction.isVisible():
                self._tableHead._tableSelectedAction.setVisible(False)
        
        if self._onSelectionChanged:
            self._onSelectionChanged(self.getselectedRowsFast())
                
                
    def _onSelectAllChanged(self, checked=None):
        if checked:
            self.selectAll() 
            if not self._tableHead._tableSelectedAction.isVisible():
                self._tableHead._tableSelectedAction.setVisible(True)
            if self._tableHead._onSelectAllRows:
                # self._tableHead._onSelectAllRows(checked, self.getselectedRowsFast())
                self._tableHead._onSelectAllRows(checked)
        else:
            self.clearSelection()
            # self._selected.setValue(False)
            if self._tableHead._tableSelectedAction.isVisible():
                self._tableHead._tableSelectedAction.setVisible(False)
        
            
    def __setup_header(self):
        self._horizontalHeader = self.horizontalHeader()

        for colIndex, headerData in enumerate(self._tableHead._headLabel):
            if headerData.get('width'):
                self._horizontalHeader.setSectionResizeMode(colIndex, QHeaderView.Fixed)
                self.setColumnWidth(colIndex, headerData.get('width'))
            else:
                if headerData.get('columnHidden') == True:
                    self.setColumnHidden(colIndex, True)
                elif headerData.get('resizeMode') == "ResizeToContents":
                    self._horizontalHeader.setSectionResizeMode(colIndex, QHeaderView.ResizeToContents)
                elif headerData.get('resizeMode') == "Stretch":
                    self._horizontalHeader.setSectionResizeMode(colIndex, QHeaderView.Stretch)
                else:
                    self._horizontalHeader.setSectionResizeMode(colIndex, QHeaderView.Interactive)


        # Thêm QCheckBox vào header của cột đầu tiên
        # self._selectAllCheckBox = Checkbox(parent=self._horizontalHeader.viewport(), checked=self._selected, size="small", onChange=self._onSelectAllChanged)
        self._selectAllCheckBox = Checkbox(parent=self._horizontalHeader.viewport(), checked=self._tableHead._numSelected, size="small", onChange=self._onSelectAllChanged)
        # self._selectAllCheckBox = Checkbox(parent=self._horizontalHeader.viewport(), checked=False, size="small", onChange=self._onSelectAllChanged)
        self._horizontalHeader.viewport().setLayout(QHBoxLayout())
        self._horizontalHeader.viewport().layout().setContentsMargins(0,0,0,0)
        if self._tableHead._tableSelectedAction:
            self._horizontalHeader.viewport().layout().addWidget(self._tableHead._tableSelectedAction)
            self._tableHead._tableSelectedAction.setVisible(False)
        self._positionHeaderCheckbox()
        # Khi cột đầu tiên được resize, cập nhật vị trí checkbox
        self._horizontalHeader.sectionResized.connect(lambda idx, oldSize, newSize: self._positionHeaderCheckbox())

    def _set_model_data(self, data=None):
        self.setModel(None)

        if not data:
            data = self._model.value

        self._abstractTableModel = AbstractTableModel(self, headLabel=self._tableHead._headLabel, data=data)
        
        self.setModel(self._abstractTableModel)
        
        self._setup_table_for_visible_indexes()

        self.__setup_header()
        
        if not data:
            self.setMaximumHeight(50) # có dòng này mới hiển thị header
        else:
            self.setMaximumHeight(16777215)
        
        self.viewport().update()

        # # 🔥 Sau khi setModel xong thì selectionModel() mới tồn tại
        if self.selectionModel():
            self.selectionModel().selectionChanged.connect(self._onSelectionChange)
        
        # model = self.model()
        # model.rowsInserted.connect(lambda *args: self.updateTableViewHeight(self))
        # model.rowsRemoved.connect(lambda *args: self.updateTableViewHeight(self))
        # model.layoutChanged.connect(lambda: self.updateTableViewHeight(self))


    def onScrollReleased(self) -> None:
        self._thread_setup_table_for_visible_indexes()

    def wheelEvent(self, event) -> None:
        super().wheelEvent(event)
        self._thread_setup_table_for_visible_indexes()
    
    def _thread_setup_table_for_visible_indexes(self):
        global SCROLL_POS

        scroll_position = self.verticalScrollBar().value()
        if SCROLL_POS == scroll_position:
            return
        SCROLL_POS = scroll_position
        self._setup_table_for_visible_indexes()

    def _setup_table_for_visible_indexes(self):
        """
        Hàm này trả về danh sách các chỉ số (index) hiện đang nằm trong vùng hiển thị của QTableView.
        """
        model = self.model()
        if not model:
            return []

        viewport_rect = self.viewport().rect()

        # Xác định hàng đầu tiên và hàng cuối cùng trong vùng nhìn thấy
        first_row = self.rowAt(viewport_rect.top())
        last_row = self.rowAt(viewport_rect.bottom())

        # Xác định cột đầu tiên và cột cuối cùng trong vùng nhìn thấy
        first_col = self.columnAt(viewport_rect.left())
        last_col = self.columnAt(viewport_rect.right())

        # Nếu không tìm thấy, set mặc định
        if first_row == -1: first_row = 0
        if last_row == -1: last_row = model.rowCount() - 1
        if first_col == -1: first_col = 0
        if last_col == -1: last_col = model.columnCount() - 1

        # Lấy danh sách index trong vùng hiển thị (dùng list comprehension)
        visible_indexes = [
            model.index(row, col)
            for row in range(first_row, last_row + 1)
            for col in range(first_col, last_col + 1)
        ]

        for index in self._last_visible_indexes:
            if index not in visible_indexes:
                self.removeIndexWidgetSignal.emit(index)
        # Gửi tín hiệu cập nhật các index hiển thị
        self.setIndexWidgetSignal.emit(visible_indexes)
        self._last_visible_indexes = visible_indexes

        # for index in range(first_row, last_row):
        #     self.setRowHeight(index, self._rowHeight)


    def _clear_index_widgets(self):
        if self.model():
            for row in range(self.model().rowCount()):
                for col in range(self.model().columnCount()):
                    index = self.model().index(row, col)
                    widget = self.indexWidget(index)
                    if widget:
                        # widget.deleteLater()  # Giải phóng bộ nhớ
                        QTimer.singleShot(110, lambda: asyncio.ensure_future(widget.deleteLater()))
                        self.setIndexWidget(index, None)

    def _remove_index_widget(self, index):
        if self.indexWidget(index):
            widget = self.indexWidget(index)
            QTimer.singleShot(10, lambda widget=widget: asyncio.ensure_future(self._rem(widget, index))) # tránh lỗi ở set_stylesheet

    async def _rem(self, widget, index):
        # self.indexWidget(index).deleteLater()
        widget.deleteLater()
        self.setIndexWidget(index, None)

    def _set_index_widget(self, indexs):

        index_rows = []
        totalRowsHeight = 0

        for index in indexs:
            index_row = index.row()
            if index_row not in index_rows:
                index_rows.append(index_row)
            if not self.indexWidget(index) and not index.parent().isValid():
                widget_info = self._model.value[index_row][index.column()]
                widget = None
                if isinstance(widget_info, Callable):
                    widget: TableViewCell = widget_info()
                    widget._indexRow = index_row
                    # print('widget_text________', widget._text)
                    widget.indexRow.connect(self._setHoverRow)
                elif isinstance(widget_info, Dict):
                    widget = widget_info["component"](**widget_info["renderProps"])
                elif isinstance(widget_info, str):
                    pass
                if isinstance(widget, QWidget):
                    self.setIndexWidget(index, widget)


        if self._maxHeight:
            for row in index_rows:
                ## self.setRowHidden(row, True)
                self.resizeRowToContents(row)
                totalRowsHeight += self.rowHeight(row)
            
            if self.height() < self._maxHeight:
                headerHeight = self.horizontalHeader().height()
                self.setFixedHeight(headerHeight + totalRowsHeight)


    ############### fluent funcs
    def isSelectRightClickedRow(self):
        return self._isSelectRightClickedRow

    def setSelectRightClickedRow(self, isSelect: bool):
        self._isSelectRightClickedRow = isSelect

    def getselectedRowsFast(self):
        selectedRows = []
        # for item in self.selectedItems():
        for item in self.selectedIndexes():
            if item.row() not in selectedRows:
                selectedRows.append(item.row())
            selectedRows.sort()
        return selectedRows


    selectRightClickedRow = Property(bool, isSelectRightClickedRow, setSelectRightClickedRow)




# iconify_cache.py
from __future__ import annotations

import os
from collections.abc import Iterator, MutableMapping
from contextlib import suppress
from pathlib import Path

_SVG_CACHE: MutableMapping[str, bytes] | None = None

PYCONIFY_CACHE: str = os.environ.get("PYCONIFY_CACHE", "")
CACHE_DISABLED: bool = PYCONIFY_CACHE.lower() in {"0", "false", "no"}


# =================================================================
# GET CACHE DIRECTORY
# =================================================================
def get_cache_directory(app_name: str = "pyconify") -> Path:
    """Return the pyconify svg cache directory."""
    if PYCONIFY_CACHE:
        return Path(PYCONIFY_CACHE).expanduser().resolve()

    if os.name == "posix":
        return Path.home() / ".cache" / app_name
    elif os.name == "nt":
        appdata = os.environ.get("LOCALAPPDATA", "~/AppData/Local")
        return Path(appdata).expanduser() / app_name

    return Path.home() / f".{app_name}"


# =================================================================
# CACHE KEY BUILDER
# =================================================================
DELIM = "_"

def cache_key(args: tuple, kwargs: dict, last_modified: int | str) -> str:
    """
    Build a Windows-safe cache key:
    prefix + name + color + width + height + flip + rotate + lastModified
    """
    parts = list(args)
    for k, v in sorted(kwargs.items()):
        if v is not None:
            parts.append(f"{k}={v}")
    parts.append(str(last_modified))

    safe = DELIM.join(map(str, parts))
    return safe.replace(":", "_").replace("/", "_").replace("\\", "_")


# =================================================================
# SVG DISK CACHE
# =================================================================
class _SVGCache(MutableMapping[str, bytes]):
    """A simple directory cache for SVG file blobs."""

    def __init__(self, directory: str | Path | None = None):
        super().__init__()
        if not directory:
            directory = get_cache_directory() / "svg_cache"
        self.path = Path(directory).expanduser().resolve()
        self.path.mkdir(parents=True, exist_ok=True)
        self._ext = ".svg"

    def path_for(self, key: str) -> Path:
        fname = f"{key}{self._ext}"
        return self.path / fname

    def __setitem__(self, key: str, value: bytes):
        self.path_for(key).write_bytes(value)

    def __getitem__(self, key: str) -> bytes:
        try:
            return self.path_for(key).read_bytes()
        except FileNotFoundError:
            raise KeyError(key)

    def __iter__(self) -> Iterator[str]:
        yield from (x.stem for x in self.path.glob(f"*{self._ext}"))

    def __delitem__(self, key: str):
        self.path_for(key).unlink(missing_ok=True)

    def __len__(self):
        return len(list(self.path.glob(f"*{self._ext}")))

    def __contains__(self, key: object) -> bool:
        return isinstance(key, str) and self.path_for(key).exists()


# =================================================================
# INIT CACHE
# =================================================================
def svg_cache() -> MutableMapping[str, bytes]:
    global _SVG_CACHE
    if _SVG_CACHE is None:
        if CACHE_DISABLED:
            _SVG_CACHE = {}
        else:
            try:
                _SVG_CACHE = _SVGCache()
            except Exception:
                _SVG_CACHE = {}

    return _SVG_CACHE


# =================================================================
# CLEAR CACHE
# =================================================================
def clear_cache() -> None:
    import shutil
    shutil.rmtree(get_cache_directory(), ignore_errors=True)
    global _SVG_CACHE
    _SVG_CACHE = None


# =================================================================
# DELETE STALE CACHE (for lastModified support)
# =================================================================
def delete_stale_svgs(prefix_last_mod: dict[str, int]):
    """
    Used externally when caller knows last_modified state.
    """
    cache = svg_cache()
    for key in list(cache):
        try:
            # key pattern: prefix_name_color..._lastModified
            parts = key.split(DELIM)
            last_mod = int(parts[-1])
            prefix = parts[0]
            if last_mod < prefix_last_mod.get(prefix, 0):
                del cache[key]
        except Exception:
            pass



from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState

from .properties_name import *
from ....system.color_manipulator import alpha


def toggle_button(_theme) -> Dict:
    theme: ThemeState = _theme
    lightMode = theme.palette.mode == 'light'
    COLORS = ['primary', 'secondary', 'info', 'success', 'warning', 'error']

    return {
        "PyToggleButton": {
            "styles": {
                "root": lambda ownerState: {
                    **{
                        "default": {
                            "selected": {
                                borderColor: theme.palette.grey._500,
                                boxShadow: f"0 0 0 0.5px {theme.palette.grey._500}",
                            }
                        },
                    },
                    **{
                        f"{_color}": {
                            "slots": {
                                "hover": {
                                    borderColor: alpha(getattr(theme.palette, _color).main, 0.48),
                                    backgroundColor: alpha(getattr(theme.palette, _color).main, theme.palette.action.hoverOpacity),
                                }
                            },
                            "props": {
                                "disabled": {
                                    "selected": {
                                        color: theme.palette.action.disabled,
                                        backgroundColor: theme.palette.action.selected,
                                        borderColor: theme.palette.action.disabledBackground,
                                    }
                                }
                            },

                        }
                        for _color in COLORS
                    }
                },
            },
        },
        "PyToggleButtonGroup": {
            "styles": {
                "root": {
                    borderRadius: theme.shape.borderRadius,
                    backgroundColor: theme.palette.background.paper,
                    border: f"solid 1px {alpha(theme.palette.grey._500, 0.08)}",
                },
                "grouped": {
                    m: 4,
                    "props": {
                        "toggleButtonSelected": {
                            boxShadow: "none",
                        },
                        "notFirstOfType": {
                            borderRadius: theme.shape.borderRadius,
                            borderColor: "transparent",
                        },
                        "notLastOfType": {
                            borderRadius: theme.shape.borderRadius,
                            borderColor: "transparent",
                        },
                    }
                },
            },
        },
    }



import uuid
from typing import Optional, Union, Dict, Callable, List
from PySide6.QtWidgets import QWidget, QHBoxLayout, QSizePolicy
from PySide6.QtCore import Qt
from qtmui.hooks import State, useEffect
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.styles import useTheme
from ..utils.validate_params import _validate_param

class DialogActions(QWidget):
    """
    A component that renders action buttons or content for dialogs, typically at the bottom.

    The `DialogActions` component is used to group action buttons (e.g., OK, Cancel) in a dialog,
    supporting all props of the Material-UI `DialogActions` component, as well as additional
    props for customization.

    Parameters
    ----------
    children : State, QWidget, List[QWidget], or None, optional
        The content of the component, typically action buttons. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    disableSpacing : State or bool, optional
        If True, removes additional margin and spacing between actions. Default is False.
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    ref : State, object, or None, optional
        A reference to the root element (custom feature, not part of Material-UI). Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QWidget` class, supporting
        props of the native component (e.g., style, className).

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `style`, `className`).
    - The `ref` parameter is a custom feature, not part of Material-UI's `DialogActions`.
    - The `children` prop must be a `QWidget`, a list of `QWidget` instances, or a `State` object.

    Demos:
    - DialogActions: https://qtmui.com/material-ui/qtmui-dialog-actions/

    API Reference:
    - DialogActions API: https://qtmui.com/material-ui/api/dialog-actions/
    """

    def __init__(
        self,
        children: Optional[Union[State, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        disableSpacing: Union[State, bool] = False,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        ref: Optional[Union[State, object]] = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.setObjectName(str(uuid.uuid4()))

        # Initialize theme
        self.theme = useTheme()

        # Store widget references to prevent Qt deletion
        self._widget_references = []

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_disableSpacing(disableSpacing)
        self._set_sx(sx)
        self._set_ref(ref)

        # Setup UI
        self.setLayout(QHBoxLayout())
        self.layout().setAlignment(Qt.AlignRight)
        self._update_spacing()
        self._add_children_to_layout()

        # Apply styles
        self._set_stylesheet()

        # Connect signals
        self._connect_signals()
        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        self.destroyed.connect(self._on_destroyed)

    # Setter and Getter methods
    # @_validate_param(file_path="qtmui.material.dialog_actions", param_name="children", supported_signatures=Union[State, QWidget, List[QWidget], type(None)])
    def _set_children(self, value):
        """Assign value to children and store widget references."""
        self._widget_references.clear()
        self._children = value
        children = self._get_children()

        if isinstance(children, QWidget):
            self._widget_references.append(children)
            children.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        elif isinstance(children, list):
            for child in children:
                if child is None:
                    continue
                if isinstance(child, QWidget):
                    self._widget_references.append(child)
                    child.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
                else:
                    raise TypeError(f"Each element in children must be a QWidget, but got {type(child)}")
        elif children is not None:
            raise TypeError(f"children must be a State, QWidget, or list of QWidgets, but got {type(children)}")

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.dialog_actions", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.dialog_actions", param_name="disableSpacing", supported_signatures=Union[State, bool])
    def _set_disableSpacing(self, value):
        """Assign value to disableSpacing."""
        self._disableSpacing = value

    def _get_disableSpacing(self):
        """Get the disableSpacing value."""
        return self._disableSpacing.value if isinstance(self._disableSpacing, State) else self._disableSpacing

    @_validate_param(file_path="qtmui.material.dialog_actions", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.dialog_actions", param_name="ref", supported_signatures=Union[State, object, type(None)])
    def _set_ref(self, value):
        """Assign value to ref (custom feature)."""
        self._ref = value

    def _get_ref(self):
        """Get the ref value."""
        return self._ref.value if isinstance(self._ref, State) else self._ref

    def forward_ref(self, ref):
        """Set the reference to the root element (custom feature)."""
        self._set_ref(ref)

    def _update_spacing(self):
        """Update layout spacing based on disableSpacing."""
        if self._get_disableSpacing():
            self.layout().setSpacing(0)
            self.layout().setContentsMargins(0, 0, 0, 0)
        else:
            self.layout().setSpacing(8)  # Default MUI spacing
            self.layout().setContentsMargins(8, 8, 8, 8)  # Default MUI padding

    def _add_children_to_layout(self):
        """Add children to the layout."""
        children = self._get_children()
        if children is None:
            return

        if isinstance(children, QWidget):
            self.layout().addWidget(children)
        elif isinstance(children, list):
            for child in children:
                if child is not None:
                    self.layout().addWidget(child)
        else:
            raise TypeError(f"children must be a State, QWidget, or list of QWidgets, but got {type(children)}")

    def _set_stylesheet(self, component_styled=None):
        """Apply styles based on theme, classes, and sx."""
        self.theme = useTheme()
        component_styled = component_styled or self.theme.components
        dialog_actions_styles = component_styled.get("DialogActions", {}).get("styles", {})
        root_styles = dialog_actions_styles.get("root", {})
        root_qss = get_qss_style(root_styles)

        # Handle sx
        sx = self._get_sx()
        sx_qss = ""
        if sx:
            if isinstance(sx, dict):
                sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
            elif isinstance(sx, Callable):
                sx_result = sx()
                if isinstance(sx_result, dict):
                    sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
                elif isinstance(sx_result, str):
                    sx_qss = sx_result
            elif isinstance(sx, str) and sx != "":
                sx_qss = sx

        # Handle classes
        classes = self._get_classes()
        classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}") if classes else ""

        # Apply MUI classes
        mui_classes = ["MuiDialogActions-root"]
        if self._get_disableSpacing():
            mui_classes.append("MuiDialogActions-spacingDisabled")

        stylesheet = f"""
            #{self.objectName()} {{
                padding: 8px;
                {root_qss}
                {classes_qss}
            }}
            {sx_qss}
        """
        self.setStyleSheet(stylesheet)

    def _connect_signals(self):
        """Connect valueChanged signals of State parameters to their slots."""
        if isinstance(self._children, State):
            self._children.valueChanged.connect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.connect(self._on_classes_changed)
        if isinstance(self._disableSpacing, State):
            self._disableSpacing.valueChanged.connect(self._on_disableSpacing_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.connect(self._on_sx_changed)
        if isinstance(self._ref, State):
            self._ref.valueChanged.connect(self._on_ref_changed)

    def _on_children_changed(self):
        """Handle changes to children."""
        self._set_children(self._children)
        self._clear_layout()
        self._add_children_to_layout()

    def _on_classes_changed(self):
        """Handle changes to classes."""
        self._set_classes(self._classes)
        self._set_stylesheet()

    def _on_disableSpacing_changed(self):
        """Handle changes to disableSpacing."""
        self._set_disableSpacing(self._disableSpacing)
        self._update_spacing()

    def _on_sx_changed(self):
        """Handle changes to sx."""
        self._set_sx(self._sx)
        self._set_stylesheet()

    def _on_ref_changed(self):
        """Handle changes to ref."""
        self._set_ref(self._ref)

    def _clear_layout(self):
        """Remove all widgets from the layout."""
        while self.layout().count():
            item = self.layout().takeAt(0)
            if item.widget():
                item.widget().setParent(None)

    def _on_destroyed(self):
        """Clean up connections when the widget is destroyed."""
        if hasattr(self, "theme"):
            self.theme.state.valueChanged.disconnect(self._set_stylesheet)
        if isinstance(self._children, State):
            self._children.valueChanged.disconnect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.disconnect(self._on_classes_changed)
        if isinstance(self._disableSpacing, State):
            self._disableSpacing.valueChanged.disconnect(self._on_disableSpacing_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.disconnect(self._on_sx_changed)
        if isinstance(self._ref, State):
            self._ref.valueChanged.disconnect(self._on_ref_changed)


from typing import Dict, Optional, Union, List, Callable, Any

from qtmui.hooks import State
from PySide6.QtWidgets import QFrame, QVBoxLayout, QWidget
from PySide6.QtCore import Qt
from ..utils.validate_params import _validate_param
from qtmui.material.styles import useTheme

class Tab(QFrame):
    """
    A tab component, styled like Material-UI Tab.

    The `Tab` component displays a tab with a label, icon, and optional content, aligning with
    MUI Tab props. Inherits from ButtonBase props.

    Parameters
    ----------
    classes : State or Dict, optional
        Override or extend styles. Default is None.
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, disables the tab. Default is False.
        Can be a `State` object for dynamic updates.
    disableFocusRipple : State or bool, optional
        If True, disables keyboard focus ripple. Default is False.
        Can be a `State` object for dynamic updates.
    disableRipple : State or bool, optional
        If True, disables the ripple effect. Default is False.
        Can be a `State` object for dynamic updates.
    icon : State, str, QWidget, or None, optional
        Icon to display. Default is None.
        Can be a `State` object for dynamic updates.
    iconPosition : State or str, optional
        Position of the icon relative to the label ('top', 'end', 'start', 'bottom').
        Default is 'top'.
        Can be a `State` object for dynamic updates.
    label : State, str, QWidget, or None, optional
        Label element. Default is None.
        Can be a `State` object for dynamic updates.
    key : State, Any, or None, optional
        Unique key for the tab (qtmui-specific). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, List, Dict, Callable, or None, optional
        System prop for CSS overrides. Default is None.
        Can be a `State` object for dynamic updates.
    value : State, Any, or None, optional
        Value of the tab. Default is None.
        Can be a `State` object for dynamic updates.
    wrapped : State or bool, optional
        If True, label can span two lines. Default is False.
        Can be a `State` object for dynamic updates.
    children : State, List[QWidget], QWidget, or None, optional
        Content of the tab (qtmui-specific, unsupported in MUI). Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to QFrame, supporting ButtonBase props.

    Notes
    -----
    - `children` is unsupported in MUI but supported in qtmui as a fallback for label.
    - `key` is a qtmui-specific feature for tab management.
    - Supports dynamic updates via State objects.
    - MUI classes applied: `MuiTab-root`.

    Demos:
    - Tab: https://qtmui.com/material-ui/qtmui-tab/

    API Reference:
    - Tab API: https://qtmui.com/material-ui/api/tab/
    """

    VALID_POSITIONS = ['top', 'end', 'start', 'bottom']

    def __init__(
        self,
        classes: Optional[Union[State, Dict, None]]=None,
        disabled: Union[State, bool]=False,
        disableFocusRipple: Union[State, bool]=False,
        disableRipple: Union[State, bool]=False,
        icon: Optional[Union[State, str, QWidget, None]]=None,
        iconPosition: Union[State, str]="top",
        label: Optional[Union[State, str, QWidget, None]]=None,
        key: Optional[Union[State, Any, None]]=None,
        sx: Optional[Union[State, List, Dict, Callable, None]]=None,
        value: Optional[Union[State, Any, None]]=None,
        wrapped: Union[State, bool]=False,
        children: Optional[Union[State, List[QWidget], QWidget, None]]=None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.setObjectName(str(id(self)))
        self.theme = useTheme()
        self._widget_references = []

        # Set properties with validation
        self._set_classes(classes)
        self._set_disabled(disabled)
        self._set_disableFocusRipple(disableFocusRipple)
        self._set_disableRipple(disableRipple)
        self._set_icon(icon)
        self._set_iconPosition(iconPosition)
        self._set_label(label)
        self._set_key(key)
        self._set_sx(sx)
        self._set_value(value)
        self._set_wrapped(wrapped)
        self._set_children(children)

        self._rendered = False


    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.tab", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.tab", param_name="disabled", supported_signatures=Union[State, bool])
    def _set_disabled(self, value):
        self._disabled = value

    def _get_disabled(self):
        return self._disabled.value if isinstance(self._disabled, State) else self._disabled

    @_validate_param(file_path="qtmui.material.tab", param_name="disableFocusRipple", supported_signatures=Union[State, bool])
    def _set_disableFocusRipple(self, value):
        self._disableFocusRipple = value

    def _get_disableFocusRipple(self):
        return self._disableFocusRipple.value if isinstance(self._disableFocusRipple, State) else self._disableFocusRipple

    @_validate_param(file_path="qtmui.material.tab", param_name="disableRipple", supported_signatures=Union[State, bool])
    def _set_disableRipple(self, value):
        self._disableRipple = value

    def _get_disableRipple(self):
        return self._disableRipple.value if isinstance(self._disableRipple, State) else self._disableRipple

    # @_validate_param(file_path="qtmui.material.tab", param_name="icon", supported_signatures=Union[State, str, QWidget, type(None)])
    def _set_icon(self, value):
        self._icon = value

    def _get_icon(self):
        return self._icon.value if isinstance(self._icon, State) else self._icon

    @_validate_param(file_path="qtmui.material.tab", param_name="iconPosition", supported_signatures=Union[State, str], valid_values=VALID_POSITIONS)
    def _set_iconPosition(self, value):
        self._iconPosition = value

    def _get_iconPosition(self):
        position = self._iconPosition.value if isinstance(self._iconPosition, State) else self._iconPosition
        return position if position in self.VALID_POSITIONS else 'top'

    @_validate_param(file_path="qtmui.material.tab", param_name="label", supported_signatures=Union[State, str, Callable, QWidget, type(None)])
    def _set_label(self, value):
        self._label = value

    def _get_label(self):
        return self._label.value if isinstance(self._label, State) else self._label

    # @_validate_param(file_path="qtmui.material.tab", param_name="key", supported_signatures=Union[State, Any, type(None)])
    def _set_key(self, value):
        self._key = value

    def _get_key(self):
        return self._key.value if isinstance(self._key, State) else self._key

    @_validate_param(file_path="qtmui.material.tab", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx

    # @_validate_param(file_path="qtmui.material.tab", param_name="value", supported_signatures=Union[State, Any, type(None)])
    def _set_value(self, value):
        self._value = value

    def _get_value(self):
        return self._value.value if isinstance(self._value, State) else self._value

    @_validate_param(file_path="qtmui.material.tab", param_name="wrapped", supported_signatures=Union[State, bool])
    def _set_wrapped(self, value):
        self._wrapped = value

    def _get_wrapped(self):
        return self._wrapped.value if isinstance(self._wrapped, State) else self._wrapped

    # @_validate_param(file_path="qtmui.material.tab", param_name="children", supported_signatures=Union[State, List[QWidget], QWidget, type(None)])
    def _set_children(self, value):
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children


    def _init_ui(self):
        if self._rendered:
            return
        self._rendered = True
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignmentFlag.AlignTop)
        self.layout().setContentsMargins(0,0,0,0)
        if self._children:
            if isinstance(self._children, list):
                for widget in self._children:
                    self.layout().addWidget(widget)
            elif isinstance(self._children, Callable):
                self.layout().addWidget(self._children())
            else:
                self.layout().addWidget(self._children)



    def generate_stylesheet(self):
        # Dùng để tạo CSS từ thuộc tính sx
        styles = []
        if isinstance(self._sx, list):
            for style_item in self._sx:
                if isinstance(style_item, dict):
                    # Thêm từng thuộc tính CSS từ dict
                    for key, value in style_item.items():
                        styles.append(f"{key}: {value};")
                elif callable(style_item):
                    # Nếu style là một hàm, gọi hàm đó để lấy giá trị CSS
                    generated_style = style_item()
                    styles.append(generated_style)
        return " ".join(styles)

    # Setter và getter cho các thuộc tính
    def set_disabled(self, disabled: bool):
        self._disabled = disabled
        self.setEnabled(not disabled)

    def set_value(self, value: Any):
        self._value = value

    def get_value(self):
        return self._value

    def set_icon(self, icon: Union[str, Any]):
        self._icon = icon
        # Cập nhật icon khi thay đổi
        if isinstance(icon, str):
            self.setIcon(icon)

    def set_label(self, label: str):
        self._label = label
        self.setText(label)

    def set_wrapped(self, wrapped: bool):
        self._wrapped = wrapped
        # Cập nhật nếu cần xử lý việc nhãn hiển thị trên nhiều dòng

    def set_icon_position(self, position: str):
        if position in ['top', 'end', 'start', 'bottom']:
            self._iconPosition = position
            # Cập nhật vị trí của icon




# qtmui/material/chart/chart_radial_bar.py
from __future__ import annotations

from typing import Any, Dict, List, Optional, Union, Callable
from math import radians
from PySide6.QtCore import Qt, QRectF
from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel
from PySide6.QtGui import (
    QPainter, QColor, QBrush, QPen, QPainterPath, QFont
)

from qtmui.material.styles import useTheme


# Widget chỉ để vẽ – không có layout → an toàn 100%
class RadialBarCanvas(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.parent_chart = parent
        self.setAttribute(Qt.WA_StyledBackground, False)

    def paintEvent(self, event):
        if not self.parent_chart:
            return

        p = QPainter(self)
        p.setRenderHint(QPainter.Antialiasing, True)

        rect = self.rect()
        side = min(rect.width(), rect.height())
        draw_rect = QRectF(
            rect.width() / 2 - side / 2,
            rect.height() / 2 - side / 2,
            side, side
        ).adjusted(20, 20, -20, -20)  # margin đẹp

        center = draw_rect.center()
        max_radius = draw_rect.width() / 2
        inner_radius = max_radius * self.parent_chart.hollow_ratio
        bar_thickness = (max_radius - inner_radius) / max(1, len(self.parent_chart._series))

        # Vẽ từng thanh + track
        for idx, value in enumerate(self.parent_chart._series):
            percentage = value / self.parent_chart.base_value
            angle_span = self.parent_chart.total_angle * percentage

            outer_r = max_radius - idx * bar_thickness
            inner_r = outer_r - bar_thickness * 0.85
            mid_r = (outer_r + inner_r) / 2

            track_rect = QRectF(center.x() - mid_r, center.y() - mid_r, mid_r * 2, mid_r * 2)

            # Track
            track_pen = QPen(QColor(self.parent_chart.track_color), outer_r - inner_r, Qt.SolidLine, Qt.RoundCap)
            p.setPen(track_pen)
            p.drawArc(track_rect.toRect(), self.parent_chart.start_angle * 16, self.parent_chart.total_angle * 16)

            # Thanh chính
            color = QColor(self.parent_chart.colors[idx % len(self.parent_chart.colors)])
            pen = QPen(color, outer_r - inner_r, Qt.SolidLine, Qt.RoundCap)
            p.setPen(pen)

            path = QPainterPath()
            path.arcMoveTo(track_rect, self.parent_chart.start_angle)
            path.arcTo(track_rect, self.parent_chart.start_angle, angle_span)
            p.strokePath(path, pen)

        # Total ở giữa
        if self.parent_chart.show_total and self.parent_chart._series:
            total_val = sum(self.parent_chart._series) if self.parent_chart._total is None else self.parent_chart._total
            total_text = (
                self.parent_chart.total_formatter() if callable(self.parent_chart.total_formatter)
                else f"{total_val:,.0f}"
            )

            p.setPen(QColor(self.parent_chart.theme.palette.text.primary))
            # p.setFont(self.parent_chart.theme.typography.h5.font())
            p.drawText(draw_rect.toRect(), Qt.AlignCenter, total_text)

            # Subtitle
            subtitle = self.parent_chart._options.get("title", {}).get("text", "")
            if subtitle:
                p.setPen(QColor(self.parent_chart.theme.palette.text.secondary))
                # p.setFont(self.parent_chart.theme.typography.body2.font())
                p.drawText(draw_rect.adjusted(0, 40, 0, -20).toRect(), Qt.AlignCenter, subtitle)

        p.end()


class ChartRadialBar(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        type: str = "radialBar",
        series: Optional[List[float]] = None,
        total: Optional[float] = None,
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        options: Optional[Dict[str, Any]] = None,
        key: Optional[str] = None,
        *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(16, 16, 16, 8)
        self.layout().setSpacing(8)

        self._series = series or []
        self._total = total
        self._options = options or {}

        if width: self.setFixedWidth(int(width))
        if height: self.setFixedHeight(int(height))
        self.setMinimumSize(200, 240)

        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self.update)

        self.labels = self._options.get("labels", [f"Series {i+1}" for i in range(len(self._series))])
        self.colors = self._options.get("colors", [
            self.theme.palette.primary.main,
            self.theme.palette.secondary.main,
            self.theme.palette.success.main,
            self.theme.palette.warning.main,
            self.theme.palette.error.main,
        ])

        # Canvas vẽ chart
        self.canvas = RadialBarCanvas(self)
        self.layout().addWidget(self.canvas)

        # Label hiển thị % dưới chart
        self.percent_label = QLabel()
        self.percent_label.setAlignment(Qt.AlignCenter)
        self.percent_label.setFont(QFont("Segoe UI", 11))
        self.layout().addWidget(self.percent_label)

        self._prepare_options()
        self._update_percent_text()

    def _prepare_options(self):
        po = self._options.get("plotOptions", {}).get("radialBar", {})
        hollow = po.get("hollow", {}).get("size", "68%")
        self.hollow_ratio = float(hollow.rstrip("%")) / 100 if isinstance(hollow, str) else 0.68
        self.track_color = po.get("track", {}).get("background", "#e0e0e033")

        value_opts = po.get("dataLabels", {}).get("value", {})
        self.show_value = value_opts.get("show", True)

        total_opts = po.get("dataLabels", {}).get("total", {})
        self.show_total = total_opts.get("show", True)
        self.total_formatter = total_opts.get("formatter")

        self.base_value = self._total or 100.0
        self.start_angle = 270
        self.total_angle = 360

    def _update_percent_text(self):
        if not self.show_value or not self._series:
            self.percent_label.setText("")
            return

        parts = []
        for i, val in enumerate(self._series):
            percent = val / self.base_value * 100
            if percent > 0:
                text = f"{percent:.1f}%"
                color = self.colors[i % len(self.colors)]
                parts.append(f'<span style="color:{color}">{text}</span>')
        self.percent_label.setText(" • ".join(parts))

    def resizeEvent(self, event):
        super().resizeEvent(event)
        self.canvas.update()
        self._update_percent_text()


import uuid
from PySide6.QtWidgets import QWidget, QHBoxLayout, QFrame
from PySide6.QtCore import Qt, Signal
from typing import Callable, Optional, Union
from qtmui.hooks import State
from qtmui.hooks import useEffect, useState

from ..stack import Stack
from ..typography import Typography
from ..button import IconButton
from ..select import Select
from ..py_iconify import Iconify
from ..textfield import TextField
from ..menu_item import MenuItem

from src.locales.translator import Translator
t = Translator()

# Định nghĩa lớp TablePagination kế thừa từ QWidget
class TablePagination(QFrame):
    # Định nghĩa tín hiệu để thông báo khi page hoặc rowsPerPage thay đổi
    pageChanged = Signal(int)  # Tín hiệu khi page thay đổi
    rowsPerPageChanged = Signal(int)  # Tín hiệu khi rowsPerPage thay đổi

    def __init__(
        self,
        count: State,  # Tổng số hàng (items)
        page: State,  # Trang hiện tại (bắt đầu từ 0)
        onPageChange: Callable[[int], None],  # Hàm callback khi page thay đổi
        rowsPerPage: State,  # Số hàng trên mỗi trang
        onRowsPerPageChange: Callable[[int], None],  # Hàm callback khi rowsPerPage thay đổi
        children: Optional[Union[list, QWidget]] = None,  # Widget cha, mặc định là None
        rowsPerPageOptions: Optional[Union[list, QWidget]] = None  # Widget cha, mặc định là None
    ) -> None:
        # Gọi hàm khởi tạo của lớp cha QWidget
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))

        # Lưu các thuộc tính vào instance
        self.count: State = count  # Tổng số hàng
        self.page: State = page  # Trang hiện tại
        self.rowsPerPage: State = rowsPerPage  # Số hàng trên mỗi trang
        self.onPageChange: Callable[[int], None] = onPageChange  # Callback khi page thay đổi
        self.onRowsPerPageChange: Callable[[int], None] = onRowsPerPageChange  # Callback khi rowsPerPage thay đổi

        def get_total_page():
            if isinstance(count, State):
                count_value = count.value
            else:
                count_value = count
            if isinstance(self.rowsPerPage, State):
                rowsPerPage_value = self.rowsPerPage.value
            else:
                rowsPerPage_value = self.rowsPerPage
            total_page = (count_value + int(rowsPerPage_value) - 1) // int(rowsPerPage_value) if int(rowsPerPage_value) > 0 else 1
            return total_page
        
        self.totalPages, self.setTotalPages = useState(get_total_page())

        # print('_____________99999999', type(self.count), type(self.rowsPerPage))
        useEffect(
            lambda: _update_ui(),
            [self.count, self.rowsPerPage]
        )

        # Tính tổng số trang

        # Thiết lập giao diện
        """Thiết lập giao diện cho TablePagination.
        Phương thức này không nhận tham số và không trả về giá trị."""
        # Tạo layout chính (ngang) cho TablePagination
        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)  # Đặt lề về 0
        layout.setSpacing(10)  # Đặt khoảng cách giữa các widget

        self.previous_enable, self.set_previous_enable = useState(self.page.value > 0)
        self.next_enable, self.set_next_enable = useState(self.page.value < self.totalPages.value - 1)


        def _handle_prev_page() -> None:
            """Xử lý sự kiện nhấn nút Previous.
            Phương thức này không nhận tham số và không trả về giá trị."""
            # Nếu không ở trang đầu, giảm page đi 1
            if self.page.value > 0:
                # Gửi tín hiệu pageChanged
                self.onPageChange(self.page.value - 1)
                # Cập nhật giao diện
            _update_ui()

        def _handle_next_page() -> None:
            """Xử lý sự kiện nhấn nút Next.
            Phương thức này không nhận tham số và không trả về giá trị."""
            # Nếu không ở trang cuối, tăng page lên 1
            print('_handel_next_page_________________', self.page.value, self.totalPages.value)
            if self.page.value < self.totalPages.value - 1:
                # Gửi tín hiệu pageChanged
                self.onPageChange(self.page.value + 1)
                # Cập nhật giao diện
            _update_ui()

        def _update_ui():
            """Cập nhật giao diện của TablePagination.
            Phương thức này không nhận tham số và không trả về giá trị."""
            self.setTotalPages(get_total_page())
            # Cập nhật nhãn hiển thị phạm vi hàng
            start = self.page.value * int(self.rowsPerPage.value) + 1
            end = min((self.page.value + 1) * int(self.rowsPerPage.value), self.count.value)
            set_range_label(f"{start}-{end} of {self.count.value}")
            # print("_update_ui_____________", self.count.value)

            # Cập nhật trạng thái nút Previous và Next
            self.set_previous_enable(self.page.value > 0)
            self.set_next_enable(self.page.value < self.totalPages.value - 1)


        # Nhãn hiển thị phạm vi hàng hiện tại
        start = self.page.value * int(self.rowsPerPage.value) + 1  # Hàng bắt đầu
        end = min((self.page.value + 1) * int(self.rowsPerPage.value), self.count.value)  # Hàng kết thúc
        range_label, set_range_label = useState(f"{start}-{end} of {self.count.value}")


        layout.addWidget(
            Stack(
                direction="row",
                alignItems="center",
                justifyContent="space-between",
                children=[
                    *(children if isinstance(children, list) else []),
                    *[
                        Stack(
                            direction="row",
                            alignItems="center",
                            justifyContent="flex-end",
                            spacing=1,
                            sx={"padding-right": 1},
                            children=[
                                # Typography(text=t.rowsPerPage),
                                Typography(text="row per page"),
                                TextField(
                                    key="iccccccccccccccccccccccc",
                                    select=True,
                                    size="small",
                                    hiddenLabel=True,
                                    value=self.rowsPerPage,
                                    # defaultValue=self.rowsPerPage.value,
                                    onChange=onRowsPerPageChange,
                                    selectOptions=[
                                        MenuItem(key=option, value=option, text=str(option))
                                        for option in rowsPerPageOptions or [5, 10, 25]
                                    ]
   
                                ),
                                Typography(text=range_label),
                                IconButton(
                                    size="small",
                                    enable=self.previous_enable,
                                    icon=Iconify(key="grommet-icons:form-previous"), 
                                    onClick=_handle_prev_page
                                ),
                                IconButton(
                                    enable=self.next_enable,
                                    size="small",
                                    icon=Iconify(key="grommet-icons:form-next"), 
                                    onClick=_handle_next_page
                                )
                            ]
                        )
                    ]
                ]
            )
        )


